import inspect
from validators import Validator, GroupValidator
from relationships import Relationship
from index import BaseKeyIndexCollectionProxy


def create(env):
    return type('Model', (BaseModel,), dict(env=env))


class ModelMeta(type):
    def __init__(cls, name, bases, dict):
        super(ModelMeta, cls).__init__(name, bases, dict)
        
        if name != "BaseModel" and name != "Model":
            cls.env.register_model(cls)
            cls._setup_inheritance()
            cls._setup_indexes(dict)
            cls._setup_relationships(dict)
            cls.validators = inspect.getmembers(cls, lambda x: isinstance(x, Validator))
            cls.relationships = inspect.getmembers(cls, lambda x: isinstance(x, Relationship))
    
    
    def _setup_inheritance(cls):
        genealogy = []
        for c in cls.mro():
            if hasattr(c, '__metaclass__') and c.__metaclass__ == ModelMeta and \
                c.__name__ != "BaseModel" and c.__name__ != "Model":
                genealogy.append(c)
        
        if len(genealogy) > 1:
            cls._type = ':'.join([c.__name__ for c in reversed(genealogy)])
            cls.ancestors = genealogy[1:]
            cls.table = cls.ancestors[0].table
        else:
            cls._type = None
            cls.ancestors = []
            cls.table = cls.env.db[cls.__name__]
        
        
    def _setup_indexes(cls, dict):
        if 'indexes' in dict:
            cls.index_definitions = dict['indexes']
        else:
            cls.index_definitions = []
            
        if cls._type:
            cls.indexes = BaseKeyIndexCollectionProxy(
                ModelIndexCollectionAdaptor(cls, cls.table.indexes),
                [('_type', cls._type)]
            )
            if tuple() not in cls.indexes:
                cls.indexes.add()
            cls.primary_index = cls.indexes.get()
        else:
            cls.indexes = ModelIndexCollectionAdaptor(cls, cls.table.indexes)
            cls.primary_index = None
        
        for fields in cls.index_definitions:
            if not isinstance(fields, tuple):
                fields = (fields,)
            if fields not in cls.indexes:
                cls.indexes.add(*fields)
            
        for a in cls.ancestors:
            for fields in a.index_definitions:
                if not isinstance(fields, tuple):
                    fields = (fields,)
                if fields not in cls.indexes:
                    cls.indexes.add(*fields)
        
        
    def _setup_relationships(cls, dict):
        for k,v in dict.items():
            if isinstance(v, Relationship):
                v.setup(k, cls)
    
        for name, inverse_rel in cls.env.backreferences.get_all(cls.__name__).items():
            rel = inverse_rel.get_inverse()
            setattr(cls, name, rel) 
            rel.setup(name, cls)
    
    
    def get(cls, id):
        if cls.primary_index:
            return cls.primary_index.get(id)
        else:
            return cls.load(cls.table.get(id))
        
        
    def cursor(cls, reverse=False):
        if cls.primary_index:
            return cls.primary_index.cursor(reverse=reverse)
        else:
            return ModelCursorAdaptor(cls, cls.table.cursor(reverse=reverse))
        
        
    def count(cls):
        if cls.primary_index:
            return cls.primary_index.count()
        else:
            return cls.table.count()
            
            
    def load(cls, data):
        if data:
            if '_type' in data:
                parts = data['_type'].split(':')
                model = cls.env.models[parts[-1]]
            else:
                model = cls
            data['_dirty'] = False
            return model(**data)


class BaseModel(object):
    
    __metaclass__ = ModelMeta
        
    
    def __init__(self, **kwargs):
        sup = super(BaseModel, self)
        sup.__setattr__('_dirty', kwargs.pop('_dirty', True))
        sup.__setattr__('_reference_fields', {})
        
        for k,v in self.validators:
            field_value = kwargs.get(k, v.default())
            sup.__setattr__(k, field_value)
        for k,v in self.relationships:
            if k in kwargs:
                self._reference_fields[k] = kwargs[k]
        
        if 'id' in kwargs:
            sup.__setattr__('id', kwargs['id'])
        else:
            sup.__setattr__('id', self.env.generate_id())
        
        self.env.instances.add(self)
        self.env.current_context().enqueue(self)
        
        
    def __setattr__(self, name, value):
        if hasattr(self.__class__, name) and isinstance(getattr(self.__class__, name), Validator):
            assert self.env.current_context().writable, "Transaction is read-only."
            oldvalue = getattr(self, name)
            if oldvalue != value:
                self._dirty = True
                super(BaseModel, self).__setattr__(name, value)
                self.env.current_context().enqueue(self)
        else:
            super(BaseModel, self).__setattr__(name, value)
        
    
    def __str__(self):
        return '<%s %s>' % (self.__class__.__name__, self.id)
    
    
    def __eq__(self, other):
        if isinstance(other, BaseModel) and other.id == self.id:
            return True
        return False
    
    
    def set_reference_field(self, name, value):
        if value != self._reference_fields.get(name):
            oldvalue = self._reference_fields.get(name)
            self._reference_fields[name] = value
            self._dirty = True
            self.env.current_context().enqueue(self)
            
            
    def get_reference_field(self, name):
        return self._reference_fields.get(name)
    
        
    def remove_reference_field(self, name):
        if name in self._reference_fields:
            self._reference_fields.pop(name)
            self._dirty = True
            self.env.current_context().enqueue(self)
        
        
    def is_dirty(self):
        return self._dirty
        
        
    def remove(self):
        for k,v in self.relationships:
            v.on_owner_remove(self)
        self.table.remove(self.id)
        self._dirty = False
        
        
    def save(self):
        if self.is_dirty():
            doc = self.validate()
            doc = self.table.save(doc)
            super(BaseModel, self).__setattr__('id', doc['id'])
        
        
    def validate(self):
        validators = {}
        values = {}
        for k,v in self.validators:
            validators[k] = v
            values[k] = getattr(self, k)
        group_validator = GroupValidator(**validators)
        validated = group_validator.validate(values)
        validated['id'] = self.id
        validated.update(self._reference_fields)
        
        if self._type:
            validated['_type'] = self._type
        
        return validated
        
        
    def to_dict(self):
        values = {}
        for k,v in self.validators:
            values[k] = getattr(self, k)
        values['id'] = self.id
        values.update(self._reference_fields)
        return values
        
        
from functools import wraps


class ModelAdaptor(object):
    
    functions = []
    iterators = []
    
    
    def __init__(self, model, adapted):
        self.model = model
        self.adapted = adapted
        
        for k in self.functions:
            setattr(self, k, self.adapt_function(getattr(self.adapted, k)))
            
        for k in self.iterators:
            setattr(self, k, self.adapt_iterator(getattr(self.adapted, k)))
        
        
    def __getattr__(self, name):
        return getattr(self.adapted, name)
            
            
    def adapt_function(self, fn):
        @wraps(fn)
        def wrapper(*args, **kwargs):
            data = fn(*args, **kwargs)
            return self.model.load(data)
        return wrapper
        
        
    def adapt_iterator(self, fn):
        @wraps(fn)
        def wrapper(*args, **kwargs):
            for data in fn(*args, **kwargs):
                yield self.model.load(data)
        return wrapper
    
    
class ModelCursorAdaptor(ModelAdaptor):
    functions = ['first', 'last']
    iterators = ['range', 'prefix', 'key']
    
    def __iter__(self):
        for data in self.adapted:
            yield self.model.load(data)
    
    
class ModelIndexAdaptor(ModelAdaptor):
    functions = ['get']
    iterators = ['all']
    
    def cursor(self, reverse=False):
        return ModelCursorAdaptor(self.model, self.adapted.cursor(reverse=reverse))
        
        
class ModelIndexCollectionAdaptor(object):
    
    def __init__(self, model, index_collection):
        self.model = model
        self.index_collection = index_collection
        
        
    def __contains__(self, fields):
        return fields in self.index_collection
        
        
    def __getitem__(self, fields):
        index = self.index_collection.__getitem__(fields)
        return ModelIndexAdaptor(self.model, index)
        
        
    def add(self, *args, **kwargs):
        self.index_collection.add(*args, **kwargs)
        
        



import threading
import dbm
from table import Table


def open(url):
    return Database(dbm.open(url))


class Database(object):
    
    def __init__(self, dbm):
        self.dbm = dbm
        self.tables = {}
        self.indexes_synced = False
        
        
    def read(self):
        self.ensure_indexes_synced()
        return DatabaseContext(self.dbm)
        
        
    def write(self):
        self.ensure_indexes_synced()
        return DatabaseContext(self.dbm, writable=True)
        
        
    def __getattr__(self, name):
        return self.get_table(name)
        
        
    def __getitem__(self, name):
        return self.get_table(name)
        
        
    def get_table(self, name):
        if name not in self.tables:
            self.tables[name] = Table(self.dbm, name)
        return self.tables[name]
        
        
    def close(self):
        self.dbm.close()
        
        
    def ensure_indexes_synced(self):
        if self.indexes_synced:
            return
        self.indexes_synced = True
        
        for table in self.tables.values():
            table.indexes.sync()
        
        
class DatabaseContext(object):
    
    def __init__(self, dbm, writable=False):
        self.dbm = dbm
        self.writable = writable
        
        
    def __enter__(self):
        self.dbm.transaction_start(writable=self.writable)
        
        
    def __exit__(self, type, value, exception):
        if value:
            self.dbm.transaction_abort()
        else:
            self.dbm.transaction_commit()
            


import database
import registry
import model
import uniqueid


def open(path):
    return Environment(path)


class Environment(object):
    
    def __init__(self, path):
        self.db = database.open(path)
        self.backreferences = registry.BackreferenceRegistry()
        self.instances = registry.ModelInstanceRegistry()
        self.models = {}
        self.context = None
        self.Model = model.create(self)
        
        
    def register_model(self, model):
        assert model.__name__ not in self.models, "Model '%s' is already defined" % model.__name__
        self.models[model.__name__] = model
        
        
    def generate_id(self):
        return uniqueid.create()
        
        
    def read(self):
        self.context = EnvironmentContext(self)
        return self.context
        
        
    def write(self):
        self.context = EnvironmentContext(self, writable=True)
        return self.context
        
        
    def current_context(self):
        assert self.context is not None, "An active transaction is required"
        return self.context
        
        
        
class EnvironmentContext(object):
    
    def __init__(self, env, writable=False):
        self.env = env
        self.writable = writable
        self.queue = {}
        self.max_queue_size = 20
        
        if writable:
            self.db_context = self.env.db.write()
        else:
            self.db_context = self.env.db.read()
        
        
    def __enter__(self):
        self.db_context.__enter__()
        
        
    def __exit__(self, type, value, traceback):
        try:
            if self.writable and not value:
                self.flush()
        except Exception, e:
            self.db_context.__exit__(e.__class__, e, None)
            raise
        else:
            self.db_context.__exit__(type, value, traceback)
        
        
    def enqueue(self, inst):
        if inst.is_dirty():
            assert self.writable, "Transaction is read-only"
        if inst.id not in self.queue:
            if len(self.queue) > self.max_queue_size:
                self.flush()
            self.queue[inst.id] = inst
        
        
    def flush(self):
        if len(self.queue) > 0:
            assert self.writable, "Transaction is read-only"
            for inst in self.queue.values():
                inst.save()
                


import weakref
import threading
from functools import partial


class ModelInstanceRegistry(object):
    
    def __init__(self):
        self._local = threading.local()
        
        
    def add(self, inst):
        id = unicode(inst.id)
        callback = partial(self._remove_model_inst_ref, id)
        ref = weakref.ref(inst, callback)
        if id not in self._get_local_instances():
            self._get_local_instances()[id] = [ref]
        else:
            self._get_local_instances()[id].append(ref)
        
        
    def __getitem__(self, id):
        id = unicode(id)
        refs = self._get_local_instances().get(id, [])
        insts = []
        for r in refs:
            inst = r()
            if inst:
                insts.append(inst)
        return insts
        
        
    def __delitem__(self, id, r):
        refs = self._get_local_instances().get(id)
        if refs:
            try:
                refs.remove(r)
            except ValueError:
                pass
            if len(refs) == 0:
                self._get_local_instances().pop(id)
    
    
    def clear(self):
        self._local = threading.local()
        
        
    def _remove_model_inst_ref(self, id, r):
        refs = self._get_local_instances().get(id)
        if refs:
            try:
                refs.remove(r)
            except ValueError:
                pass
            if len(refs) == 0:
                self._get_local_instances().pop(id)
                
                
    def _get_local_instances(self):
        if not hasattr(self._local, 'instances'):
            self._local.instances = {}
        return self._local.instances
    
    
    
class BackreferenceRegistry(object):
    
    def __init__(self):
        self._backrefs = {}
        
        
    def get(self, model_name, rel_name):
        if model_name not in self._backrefs or rel_name not in self._backrefs[model_name]:
            return None
        return self._backrefs[model_name][rel_name]
        
        
    def get_all(self, model_name):
        return self._backrefs.get(model_name, {})
        
        
    def set(self, model_name, rel_name, rel):
        if model_name not in self._backrefs:
            self._backrefs[model_name] = {}
        self._backrefs[model_name][rel_name] = rel
        


import unittest
import os.path
import shutil
import threading
from handbag import database

TEST_PATH = "/tmp/handbag-test.db"
TEST_URL = "lmdb://%s" % TEST_PATH


class TestTable(unittest.TestCase):
    
    def setUp(self):
        if os.path.exists(TEST_PATH):
            shutil.rmtree(TEST_PATH)
        self.db = database.open(TEST_URL)
        
        
    def test_add(self):
        foos = self.db.foos
        
        with self.db.write():
            foo = foos.save({'skidoo':23})
        
        with self.db.read():
            foo = foos.get(foo['id'])
            self.assertEqual(foo['skidoo'], 23)
        
        
    def test_update(self):
        foos = self.db.foos
        
        with self.db.write():
            foo = foos.save({'skidoo':23})
        
        with self.db.read():
            foo = foos.get(foo['id'])
            self.assertEqual(foo['skidoo'], 23)
            
        with self.db.write():
            foo['skidoo'] = 24
            foos.save(foo)
            
        with self.db.read():
            foo = foos.get(foo['id'])
            self.assertEqual(foo['skidoo'], 24)
        
        
    def test_remove(self):
        foos = self.db.foos
        
        with self.db.write():
            foo = foos.save({'skidoo':23})
        
        with self.db.read():
            foo = foos.get(foo['id'])
            self.assertEqual(foo['skidoo'], 23)
            
        with self.db.write():
            foos.remove(foo['id'])
        
        with self.db.read():
            foo = foos.get(foo['id'])
            self.assertEqual(foo, None)
        
        
    def test_count(self):
        foos = self.db.foos
        
        with self.db.write():
            foo = foos.save({'skidoo':23})
            
        with self.db.read():
            self.assertEqual(foos.count(), 1)
            
        with self.db.write():
            for i in range(0,10):
                foos.save({'skidoo': i * 23})
        
        with self.db.read():
            self.assertEqual(foos.count(), 11)
        
        
    def test_table_threads(self):
        foos = self.db.foos
        
        def add_one():
            with self.db.write():
                foos.save({'foo':'bar'})
        
        t1 = threading.Thread(target=add_one)
        t2 = threading.Thread(target=add_one)
        t1.start()
        t2.start()
        t1.join()
        t2.join()
        
        with self.db.read():
            self.assertEqual(foos.count(), 2)


import unittest
import os.path
import shutil
import string
from handbag import database

TEST_PATH = "/tmp/handbag-test.db"
TEST_URL = "lmdb://%s" % TEST_PATH


class TestIndex(unittest.TestCase):
    
    def setUp(self):
        if os.path.exists(TEST_PATH):
            shutil.rmtree(TEST_PATH)
        self.db = database.open(TEST_URL)
                
                
    def test_create(self):
        foos = self.db.foos
        foos.indexes.add('skidoo')
        
        with self.db.write():
            for i in range(0,20):
                foos.save({'skidoo':i,'foo':'bar'})
        
        with self.db.read():
            result = foos.indexes['skidoo'].get({'skidoo':5})
            self.assertEqual(result['skidoo'], 5)
            
            
    def test_remove(self):
        foos = self.db.foos
        foos.indexes.add('skidoo')
        
        with self.db.write():
            for i in range(0,20):
                foos.save({'skidoo':i,'foo':'bar'})
        
        with self.db.read():
            result = foos.indexes['skidoo'].get({'skidoo':5})
            self.assertEqual(result['skidoo'], 5)
            
        with self.db.write():
            foos.remove(result['id'])
            
        with self.db.read():
            result = foos.indexes['skidoo'].get({'skidoo':5})
            self.assertEqual(result, None)
            
            
    def test_dupes(self):
        foos = self.db.foos
        foos.indexes.add('skidoo')
        
        with self.db.write():
            foos.save({'skidoo':23,'foo':'bar'})
            foos.save({'skidoo':23,'foo':'baz'})
        
        with self.db.read():
            result = foos.indexes['skidoo'].get({'skidoo':23})
            self.assertEqual(result['foo'], 'bar')
            
            results = list(foos.indexes['skidoo'].all({'skidoo':23}))
            self.assertEqual(len(results), 2)
            self.assertEqual(results[0]['foo'], 'bar')
            self.assertEqual(results[1]['foo'], 'baz')
            
        with self.db.write():
            foos.remove(results[0]['id'])
            
        with self.db.read():
            results = list(foos.indexes['skidoo'].all({'skidoo':23}))
            self.assertEqual(len(results), 1)
            self.assertEqual(results[0]['foo'], 'baz')
            
            
    def test_list(self):
        foos = self.db.foos
        foos.indexes.add('things')
        
        with self.db.write():
            foos.save({'things':[1,2,3]})
            foos.save({'things':['a','b','c']})
            
            
        with self.db.read():
            result = foos.indexes['things'].get({'things':[1,2,3]})
            self.assertEqual(result['things'], [1,2,3])
            result = foos.indexes['things'].get({'things':2})
            self.assertEqual(result['things'], [1,2,3])
            
            
    def test_dict(self):
        foos = self.db.foos
        foos.indexes.add('thing.name')
        
        with self.db.write():
            foos.save({'thing':{'name':'foo'}, 'skidoo':23})
            foos.save({'thing':{'name':'bar'}, 'skidoo':24})
            
        with self.db.read():
            result = foos.indexes['thing.name'].get({'thing.name':'bar'})
            self.assertEqual(result['skidoo'], 24)
            result = foos.indexes['thing.name'].get({'thing.name':'foo'})
            self.assertEqual(result['skidoo'], 23)
            
            
    def test_cursor(self):
        foos = self.db.foos
        foos.indexes.add('skidoo')
        foo_list = []
        
        with self.db.write():
            for i in range(0,20):
                foo = foos.save({'skidoo':i,'foo':'bar'})
                foo_list.append(foo)
                
        with self.db.read():
            cur = foos.indexes['skidoo'].cursor()
            results = list(cur.range({'skidoo':5}, {'skidoo':13}))
            self.assertEqual(results, foo_list[5:13])
            
            
    def test_count(self):
        foos = self.db.foos
        foos.indexes.add('skidoo')
        
        with self.db.write():
            for i in range(0,20):
                foos.save({'skidoo':i})
                
        with self.db.read():
            self.assertEqual(foos.indexes['skidoo'].count(), 20)
            
            
    def test_sync(self):
        foos = self.db.foos
        foos.indexes.add('skidoo')
        
        with self.db.write():
            for i in range(0,20):
                foos.save({'skidoo':i})
        
        self.db.close()
        self.db = database.open(TEST_URL)
        foos = self.db.foos
        foos.indexes.add('skidoo')
        
        with self.db.read():
            self.assertEqual(foos.indexes['skidoo'].count(), 20)
            
            
    def test_fn_keys(self):
        def gen_keys(doc):
            if 'skidoo' in doc:
                return [doc['skidoo'] * 100]
            else:
                return []
            
        foos = self.db.foos
        foos.indexes.add(('times-one-hundred', gen_keys))
        
        with self.db.write():
            for i in range(1,21):
                foos.save({'skidoo':i})
        
        with self.db.read():
            doc = foos.indexes['times-one-hundred'].get({'times-one-hundred':300})
            self.assertEqual(doc['skidoo'], 3)
                    
                    
    def test_filter(self):
        def greater_than_ten(doc):
            return doc.get('skidoo', 0) > 10
            
        foos = self.db.foos
        foos.indexes.add('skidoo', filter=greater_than_ten)
        
        with self.db.write():
            for i in range(0,20):
                foos.save({'skidoo':i})
                
        with self.db.read():
            self.assertEqual(foos.indexes['skidoo'].count(), 9)
            


"""Similar to, but incompatible with BSON"""

import struct
import cStringIO
import calendar
import pytz
from datetime import datetime


class DecodeError(Exception):
    pass
    
    
class EncodeError(Exception):
    pass


def dumps(value):
    buff = cStringIO.StringIO()
    encode(value, buff)
    return buff.getvalue()
    
    
def dump(value, stream):
    encode(value, stream)
    
    
def loads(bytes):
    buff = cStringIO.StringIO(bytes)
    return load(buff)
    
    
def load(stream):
    return decode(stream)
    

_type_by_magic = {
    '\x01': "dict",
    '\x02': "list",
    '\x03': "double",
    '\x04': "unicode",
    '\x05': "binary",
    '\x06': "bool",
    '\x07': "datetime",
    '\x08': "int32",
    '\x09': "int64",
    '\x0A': "none"
}


_magic_by_type = {}
for k,v in _type_by_magic.items():
    _magic_by_type[v] = k
    
    
_encode_by_type = {}
_decode_by_type = {}



def dumpone(value):
    type_name = None
    if isinstance(value, float):
        type_name = 'double'
    elif isinstance(value, unicode):
        type_name = 'unicode'
    elif isinstance(value, str):
        type_name = 'binary'
    elif isinstance(value, bool):
        type_name = 'bool'
    elif isinstance(value, datetime):
        type_name = 'datetime'
    elif isinstance(value, int):
        if value < -0x80000000 or value > 0x7fffffff:
            type_name = 'int64'
        else:
            type_name = 'int32'
    elif isinstance(value, long):
        type_name = 'int64'
    elif value is None:
        type_name = 'none'
    elif isinstance(value, dict):
        type_name = 'dict'
    elif isinstance(value, (list, tuple)):
        type_name = 'list'
    else:
        raise EncodeError, "I can't encode a single '%s'" % str(value)
        
    encode_fn = _encode_by_type[type_name]
    
    if type_name in ('dict', 'list'):
        stream = cStringIO.StringIO()
        encode_fn(value, stream)
        bytes = stream.getvalue()
    else:
        bytes = encode_fn(value)
        
    magic = _magic_by_type[type_name]
    return magic + bytes
    
    
def loadone(bytes):
    if len(bytes) == 0:
        raise DecodeError, "Can't decode the empty string"
    magic = bytes[0]
    type_name = _type_by_magic.get(magic)
    if not type_name:
        raise DecodeError, "Unknown magic number %s" % str(magic).encode('string-escape')
    decode_fn = _decode_by_type[type_name]
    return decode_fn(bytes[1:])
    


def encode(value, stream):
    type_name = None
    if isinstance(value, float):
        type_name = 'double'
    elif isinstance(value, unicode):
        type_name = 'unicode'
    elif isinstance(value, str):
        type_name = 'binary'
    elif isinstance(value, bool):
        type_name = 'bool'
    elif isinstance(value, datetime):
        type_name = 'datetime'
    elif isinstance(value, int):
        if value < -0x80000000 or value > 0x7fffffff:
            type_name = 'int64'
        else:
            type_name = 'int32'
    elif isinstance(value, long):
        type_name = 'int64'
    elif value is None:
        type_name = 'none'
        
    if type_name is not None:
        encode_fn = _encode_by_type[type_name]
        bytes = encode_fn(value)
        length = len(bytes)
        magic = _magic_by_type[type_name]
        stream.write(magic)
        stream.write(struct.pack('>I', length))
        stream.write(bytes)
    elif isinstance(value, dict):
        stream.write(_magic_by_type['dict'])
        encode_dict(value, stream)
    elif isinstance(value, (list, tuple)):
        stream.write(_magic_by_type['list'])
        encode_list(value, stream)
    else:
        raise EncodeError, "I have no idea how to encode '%s'" % str(value)


class StopDecoding(Exception):
    pass

        
def decode(stream):
    type_name = read_type(stream)
    
    if type_name == 'dict':
        return decode_dict(stream)
    elif type_name == 'list':
        return decode_list(stream)
    else:
        decode_fn = _decode_by_type[type_name]
        data = read_chunk(stream)
        return decode_fn(data)
        
        
def read_type(stream):
    magic = stream.read(1)
    if not magic:
        raise StopDecoding
    if magic == '\x00':
        raise StopDecoding
    type_name = _type_by_magic.get(magic)
    if not type_name:
        raise DecodeError, "Unknown magic number %s" % str(magic).encode('string-escape')
    return type_name
    
    
def read_chunk(stream):
    data = stream.read(struct.calcsize('>I'))
    if not data:
        raise StopDecoding
    length = struct.unpack('>I', data)[0]
    if length == 0:
        return ''
    data = stream.read(length)
    if not data:
        raise StopDecoding
    if len(data) < length:
        raise ValueError, "Unexpected end of stream"
    return data


def encode_double(value):
    return struct.pack('>d', value)
    
    
def decode_double(bytes):
    return struct.unpack('>d', bytes)[0]
    
    
def encode_unicode(value):
    return value.encode('utf8')
    
    
def decode_unicode(bytes):
    return bytes.decode('utf-8')
    
    
def encode_binary(value):
    return value
    
    
def decode_binary(bytes):
    return bytes
    
    
def encode_bool(value):
    return struct.pack('>b', 1 if value else 0)
    
    
def decode_bool(bytes):
    value = struct.unpack('>b', bytes)[0]
    return bool(value)
    
    
def encode_datetime(value):
    ms = calendar.timegm(value.utctimetuple()) * 1000 + value.microsecond / 1000.0
    return struct.pack('>d', ms)
    
    
def decode_datetime(bytes):
    ms = struct.unpack('>d', bytes)[0]
    return datetime.fromtimestamp(ms / 1000.0, pytz.utc)
    
    
def encode_int32(value):
    return struct.pack('>i', value)
    
    
def decode_int32(bytes):
    return struct.unpack('>i', bytes)[0]
    
    
def encode_int64(value):
    return struct.pack('>q', value)
    
    
def decode_int64(bytes):
    return struct.unpack('>q', bytes)[0]
    
    
def encode_none(value):
    return '\x00'
    
    
def decode_none(bytes):
    return None
    
    
def encode_dict(value, stream):
    for k,v in value.items():
        encode(k, stream)
        encode(v, stream)
    stream.write('\x00')
        
        
def decode_dict(stream):
    d = {}
    while True:
        try:
            k = decode(stream)
            d[k] = decode(stream)
        except StopDecoding:
            break
    return d
    
    
def encode_list(value, stream):
    for v in value:
        encode(v, stream)
    stream.write('\x00')
    
    
def decode_list(stream):
    l = []
    while True:
        try:
            l.append(decode(stream))
        except StopDecoding:
            break
    return l
    
    
_locals = locals()
for k in _magic_by_type.keys():
    _encode_by_type[k] = _locals['encode_%s' % k]
    _decode_by_type[k] = _locals['decode_%s' % k]
    


from index import BaseKeyIndexCollectionProxy

__all__ = [
    'One',
    'OneToOne',
    'ManyToOne',
    'OneToMany',
    'ManyToMany',
    'Relationship'
]


class Relationship(object):
    
    def __init__(self, target_model, inverse=None, cascade=False):
        """Create a relationship between two models.
        
        :param target_model: The model type that is the target of the relationship.
        :param inverse: The name of the inverse relationsip on the target. This relationship will be created automatically and does not need to be defined on the target.
        :param cascade: If ``True``, when an instance is removed, the related target models will also be removed.
        """
        self.name = None
        self.model = None
        self.env = None
        self.is_setup = False
        self._target_model = target_model
        self._inverse_name = inverse
        self._inverse = None
        self._cascade = cascade
        
        
    def setup(self, name, model):
        if self.is_setup:
            return
        self.is_setup = True
        self.name = name
        self.model = model
        self.env = model.env
        self.resolve_backreferences()
        
        
    def resolve_backreferences(self):
        if not self._inverse_name:
            return
        
        if self.is_target_model_defined():
            backref = self.env.backreferences.get(self.model.__name__, self.name)
            if backref:
                self.set_inverse(backref)
            else:
                if hasattr(self._target_model, self._inverse_name):
                    inverse_rel = getattr(self._target_model, self._inverse_name)
                    assert not inverse_rel.has_inverse(), \
                        "Attempt to redefine inverse relationship %s.%s" % (self._target_model.__name__, self._inverse_name)
                else:
                    inverse_rel = self.get_inverse()
                    setattr(self._target_model, inverse_rel.name, inverse_rel)
                inverse_rel.set_inverse(self)
        else:
            assert self.env.backreferences.get(self.get_target_model_name(), self._inverse_name) is None, \
                "Attempt to redefine inverse relationship %s.%s" % (self.get_target_model_name(), self._inverse_name)
            self.env.backreferences.set(self.get_target_model_name(), self._inverse_name, self)
        
        
    def set_inverse(self, rel):
        if not isinstance(rel, Relationship):
            raise TypeError, "Expected a Relationship instance."
        self.validate_inverse(rel)
        self._inverse = rel
        rel._inverse = self
        self.on_inverse_set()
        rel.on_inverse_set()
        
        
    def get_inverse(self):
        if not self._inverse:
            self._inverse = self.create_inverse()
            self._inverse._target_model = self.model
            self._inverse.model = self.get_target_model()
            self._inverse._inverse_name = self.name
            self._inverse._inverse = self
            self._inverse.name = self._inverse_name
        return self._inverse
        
        
    def has_inverse(self):
        return self._inverse is not None
        
        
    def validate_inverse(self, rel):
        inverse_rel = self.create_inverse()
        if rel.__class__ != inverse_rel.__class__:
            raise TypeError, "Expected %s to be of type %s" % (rel.get_full_name(), inverse_rel.__class__.__name__)
        if rel.get_target_model() != inverse_rel.get_target_model():
            raise TypeError, "Expected %s to have a target of type %s" % (rel.get_full_name(), inverse_rel.get_target_model_name())
        
        
    def create_inverse(self):
        raise NotImplementedError
        
        
    def on_owner_remove(self, owner):
        if self._cascade:
            self.cascade(owner)
            
            
    def on_inverse_set(self):
        pass
        
        
    def is_target_model_defined(self):
        return not isinstance(self._target_model, basestring)
        
        
    def get_target_model_name(self):
        if self.is_target_model_defined():
            return self._target_model.__name__
        else:
            return self._target_model
        
        
    def get_target_model(self):
        if not self.is_target_model_defined():
            name = self.get_target_model_name()
            try:
                self._target_model = self.env.models[name]
            except KeyError:
                raise Exception, "Could not find a model called '%s'" % name
        return self._target_model
        
        
    def get_full_name(self):
        return "%s.%s" % (self.model.__name__, self.name)
        
        
    def __get__(self, instance, cls):
        if instance:
            return self.get(instance)
        else:
            return self
            
            
    def __set__(self, instance, value):
        model = self.get_target_model()
        assert isinstance(value, model), "Expected an instance of %s" % model.__name__
        self.set(instance, value)
        
        
    def get(self, owner):
        raise NotImplementedError
        
        
    def set(self, owner, target, update_inverse=True):
        raise NotImplementedError
        
        
    def cascade(self, owner):
        raise NotImplementedError



class One(Relationship):
    
    def __init__(self, target_model, inverse=None):
        if inverse is not None:
            raise TypeError, "One-type relationships don't have an inverse."
        super(One, self).__init__(target_model)
        
        
    def get(self, owner):
        if not hasattr(self, '_cache'):
            id = owner.get_reference_field(self.name)
            if id:
                self._cache = self.get_target_model().get(id)
            else:
                self._cache = None
        return self._cache
        
        
    def set(self, owner, target, update_inverse=True):
        self._cache = target
        
        for obj in self.env.instances[owner.id]:
            obj.set_reference_field(self.name, target.id)
        
        
    def cascade(self, owner):
        target = self.get(owner)
        if target:
            target.remove()

        
class OneToOne(One):
    
    def __init__(self, target_model, inverse=None, cascade=False):
        Relationship.__init__(self, target_model, inverse, cascade)
        
        
    def create_inverse(self):
        return OneToOne(self.model)
        
        
    def set(self, owner, target, update_inverse=True):
        One.set(self, owner, target)
        if update_inverse and self._inverse:
            self._inverse.set(target, owner, update_inverse=False)
            
            
class ManyToOne(One):
    
    def __init__(self, target_model, inverse=None):
        Relationship.__init__(self, target_model, inverse)
        
    
    def create_inverse(self):
        return OneToMany(self.model)


class Many(Relationship):
    
    
    def __init__(self, *args, **kwargs):
        self.indexes = kwargs.pop('indexes', [])
        super(Many, self).__init__(*args, **kwargs)
    
    
    def setup(self, name, model):
        if not self._inverse_name:
            self._inverse_name = model.__name__.lower()
            
        super(Many, self).setup(name, model)
        
        
    def on_inverse_set(self):
        self.setup_indexes()
        
        
    def setup_indexes(self):
        raise NotImplementedError
        
        
    def iter(self, owner):
        raise NotImplementedError
        
        
    def get(self, owner):
        return ManyProxy(self, owner)
        
        
    def cascade(self, owner):
        self.remove(owner)
        
        
    def add(self, owner, target):
        raise NotImplementedError
        
        
    def remove(self, owner, target):
        raise NotImplementedError
        
        
    def count(self, owner):
        raise NotImplementedError
        
        
    def contains(self, owner, obj):
        raise NotImplementedError
        
        
    def base_index_key(self, owner):
        raise NotImplementedError


class ManyProxy(object):
    
    def __init__(self, rel, owner):
        self._rel = rel
        self._owner = owner
        self.indexes = BaseKeyIndexCollectionProxy(
            self._rel.get_target_model().indexes,
            self._rel.base_index_key(owner)
        )
        
    def count(self):
        return self._rel.count(self._owner)
        
        
    def __iter__(self):
        return self._rel.iter(self._owner)
        
        
    def __contains__(self, obj):
        return self._rel.contains(self._owner, obj)
        
        
    def __len__(self):
        return self.count()
        
        
    def first(self):
        return self.__iter__().next()
        
        
    def add(self, target):
        return self._rel.add(self._owner, target)
        
        
    def remove(self, target):
        return self._rel.remove(self._owner, target)
    
    
class OneToMany(Many):
    
    def setup_indexes(self):
        index_collection = self.get_target_model().indexes
        index_collection.add(self._inverse_name)
        
        for fields in self.indexes:
            if isinstance(fields, tuple):
                complete_fields = (self._inverse_name,) + fields
            else:
                complete_fields = (self._inverse_name, fields)
            index_collection.add(*complete_fields)
        
    
    def iter(self, owner):
        return self.get_target_model().indexes[self._inverse_name].cursor().key(owner.id)
    
    
    def create_inverse(self):
        return ManyToOne(self.model)
        
        
    def add(self, owner, target):
        target.set_reference_field(self._inverse_name, owner.id)
        
        
    def remove(self, owner, target):
        target.remove_reference_field(self._inverse_name)
        
        
    def count(self, owner):
        return self.get_target_model().indexes[self._inverse_name].cursor().count_key(owner.id)
        
        
    def cascade(self, owner):
        for target in list(self.iter(owner)):
            target.remove()
            
            
    def contains(self, owner, obj):
        if isinstance(obj, self.get_target_model()):
            return obj.get_reference_field(self._inverse_name) == owner.id
        else:
            return False
            
            
    def base_index_key(self, owner):
        return [(self._inverse_name, owner.id)]

    
class ManyToMany(Many):
    
    def setup_indexes(self):
        self.full_name = "%s.%s" % (self.model.__name__, self.name)
        self.full_inverse_name = "%s.%s" % (self.get_target_model_name(), self._inverse_name)
        join_table_name = ','.join(sorted([self.full_name, self.full_inverse_name]))
        self.join_table = self.model.env.db[join_table_name]
        self.model_names = tuple(sorted([self.model.__name__, self.get_target_model_name()]))
        if self.model_names not in self.join_table.indexes:
            self.join_table.indexes.add(*self.model_names)
            self.join_table.indexes.add(self.model_names[0])
            self.join_table.indexes.add(self.model_names[1])
        
        target_model = self.get_target_model()
        for fields in self.indexes:
            if not isinstance(fields, tuple):
                fields = (fields,)
            fields = ( (self.full_name, self.get_owner_ids), ) + fields
            target_model.indexes.add(*fields)
            
    
    def get_owner_ids(self, doc):
        owner_ids = []
        for pair in self.join_table.indexes[self.get_target_model_name()].all(doc['id']):
            owner_ids.append(pair[self.model.__name__])
        return owner_ids
        
        
    def iter(self, owner):
        target_model = self.get_target_model()
        for doc in self.join_table.indexes[self.model.__name__].all(owner.id):
            inst = target_model.get(doc[target_model.__name__])
            if inst:
                yield inst
            else:
                self.join_table.remove(doc['id'])
        
    
    def create_inverse(self):
        return ManyToMany(self.model)
        
        
    def validate_inverse(self, rel):
        super(ManyToMany, self).validate_inverse(rel)
        assert isinstance(rel, ManyToMany), "Expected ManyToMany got %s" % rel.__class__.__name__
        
        
    def add(self, owner, target):
        if not self.contains(owner, target):
            self.join_table.save(self.make_join_doc(owner, target))
        
        
    def remove(self, owner, target):
        key = self.make_join_doc(owner, target)
        existing = self.join_table.indexes[self.model_names].get(key)
        if existing:
            self.join_table.remove(existing['id'])
        
        
    def cascade(self, owner):
        for target in self.iter(owner):
            target.remove()
        for doc in self.join_table.indexes[self.model.__name__].all(owner.id):
            self.join_table.remove(doc['id'])
        
        
    def count(self, owner):
        return self.join_table.indexes[self.model.__name__].cursor().count_key(owner.id)
        
        
    def contains(self, owner, obj):
        if isinstance(obj, self.get_target_model()):
            key = self.make_join_doc(owner, obj)
            return self.join_table.indexes[self.model_names].cursor().count_key(key)
        else:
            return False
        
        
    def base_index_key(self, owner):
        return [(self.full_name, owner.id)]
        
        
    def make_join_doc(self, owner, target):
        return {
            self.model.__name__: owner.id,
            self.get_target_model_name(): target.id
        }
        


import lmdb
import threading
from abstract import AbstractDBM, AbstractDBMCursor

class LMDBDBM(AbstractDBM):
    
    def __init__(self, url):
        self._path = url.path
        self._env = None
        self._dbs = {}
        self._local = threading.local()
        
        
    def add_namespace(self, namespace, duplicate_keys=False):
        assert self._env is None, "Can't add a namespace after the environment has been opened"
        if namespace not in self._dbs:
            self._dbs[namespace] = {
                'duplicate_keys': duplicate_keys
            }
        
    
    def transaction_start(self, writable=False):
        if len(self._get_local_transactions()) > 0:
            parent = self._get_local_transactions()[-1][1]
        else:
            parent = None
        txn = lmdb.Transaction(self._get_env(), write=writable, parent=parent)
        self._get_local_transactions().append((writable, txn))
        return txn
        
        
    def transaction_commit(self):
        self._require_transaction()
        txn = self._get_local_transactions().pop()[1]
        txn.commit()
        
        
    def transaction_abort(self):
        self._require_transaction()
        txn = self._get_local_transactions().pop()[1]
        txn.abort()
        
        
    def is_transaction_writable(self):
        self._require_transaction()
        return self._get_local_transactions()[-1][0]
        
        
    def put(self, namespace, key, value):
        db = self._dbs[namespace]
        txn = self._current_transaction()
        txn.put(key, value, db=db)
        
        
    def delete(self, namespace, key, value=None):
        if value is None:
            value = ''
        db = self._dbs[namespace]
        txn = self._current_transaction()
        txn.delete(key, value=value, db=db)
        
        
    def delete_all(self, namespace):
        db = self._dbs[namespace]
        txn = self._current_transaction()
        txn.drop(db, delete=False)
        
        
    def get(self, namespace, key):
        db = self._dbs[namespace]
        txn = self._current_transaction()
        value = txn.get(key, db=db)
        return value
        
        
    def cursor(self, namespace):
        db = self._dbs[namespace]
        txn = self._current_transaction()
        cur = txn.cursor(db=db)
        return LMDBCursor(cur)
        
        
    def count(self, namespace):
        db = self._dbs[namespace]
        txn = self._current_transaction()
        return txn.stat(db)['entries']
        
        
    def close(self):
        if self._env:
            self._env.close()
        
        
    def _current_transaction(self):
        self._require_transaction()
        return self._get_local_transactions()[-1][1]
        
        
    def _require_transaction(self):
        assert len(self._get_local_transactions()) > 0, "An active transaction is required"
        
        
    def _get_env(self):
        if not self._env:
            self._env = lmdb.Environment(self._path, subdir=True, map_size=2147483648, max_dbs=len(self._dbs))
            for name,options in self._dbs.items():
                self._dbs[name] = self._env.open_db(name, 
                    dupsort=options.get('duplicate_keys', False))
        return self._env
        
    def _get_local_transactions(self):
        if not hasattr(self._local, 'transactions'):
            self._local.transactions = []
        return self._local.transactions
        
        
class LMDBCursor(AbstractDBMCursor):
    
    def __init__(self, dbm_cur):
        super(LMDBCursor, self).__setattr__('_dbm_cur', dbm_cur)
        
        
    def __getattribute__(self, name):
        if name == 'jump':
            return super(LMDBCursor, self).__getattribute__('_dbm_cur').set_range
        return getattr(super(LMDBCursor, self).__getattribute__('_dbm_cur'), name)
        


from urlparse import urlparse
from lmdbdbm import LMDBDBM

__all__ = ['backends', 'open']

backends = {
    'lmdb': LMDBDBM
}


def open(url):
    parsed_url = urlparse(url)
    if parsed_url.scheme not in backends:
        raise Exception, "No backend found for scheme '%s'" % parsed_url.scheme
    dbm_cls = backends[parsed_url.scheme]
    return dbm_cls(parsed_url)


class Transaction(object):
    
    def __init__(self, dbm, write=False):
        self.dbm = dbm
        self.write = write
        
        
    def __enter__(self):
        self.dbm_txn = self.dbm.transaction(write=self.write)
        return self
        
        
    def __exit__(self, type, value, traceback):
        if value:
            self.dbm_txn.abort()
        else:
            self.dbm_txn.commit()


import unittest
import os.path
import shutil
from handbag import environment
from handbag.validators import *

TEST_PATH = "/tmp/handbag-test.db"
TEST_URL = "lmdb://%s" % TEST_PATH


class TestModel(unittest.TestCase):
    
    def setUp(self):
        if os.path.exists(TEST_PATH):
            shutil.rmtree(TEST_PATH)
        self.env = environment.open(TEST_URL)
        
        
    def test_retrieve(self):
        class Foo(self.env.Model):
            name = Text()
        
        with self.env.write():
            foo = Foo(name="I'm boring today")
            foo_id = foo.id
        
        with self.env.read():
            foo = Foo.get(foo_id)
            self.assertEquals(foo.name, "I'm boring today")
            
            
    def test_modify(self):
        class Foo(self.env.Model):
            name = Text()
        
        with self.env.write():
            foo = Foo(name="I'm boring today")
            foo_id = foo.id
        
        with self.env.read():
            foo = Foo.get(foo_id)
            self.assertEquals(foo.name, "I'm boring today")
            
        with self.env.write():
            foo = Foo.get(foo_id)
            foo.name = 'Excitement Pants!'
            
        with self.env.read():
            foo = Foo.get(foo_id)
            self.assertEquals(foo.name, 'Excitement Pants!')
            
            
    def test_remove(self):
        class Foo(self.env.Model):
            name = Text()
            
        with self.env.write():
            foo = Foo(name="Snu Fu")
            foo_id = foo.id
        
        with self.env.read():
            foo = Foo.get(foo_id)
            self.assertEquals(foo.name, "Snu Fu")
        
        with self.env.write():
            foo = Foo.get(foo_id)
            foo.remove()
            
        with self.env.read():
            foo = Foo.get(foo_id)
            self.assertEquals(foo, None)
            
        
    def test_abort_on_fail(self):
        class Foo(self.env.Model):
            name = Text()
        
        foo_id = None
        with self.assertRaises(InvalidGroupError):
            with self.env.write():
                foo = Foo()
                foo_id = foo.id
        
        with self.env.read():
            self.assertEquals(Foo.get(foo_id), None)
        
        
    def test_read_fail_outside_txn(self):
        class Foo(self.env.Model):
            pass
        
        with self.env.write():
            foo = Foo()
            foo_id = foo.id
        
        with self.assertRaises(AssertionError):
            foo = Foo.get(foo_id)
        
            
    def test_write_fail_outside_txn(self):
        class Foo(self.env.Model):
            name = Text()
            
        with self.assertRaises(AssertionError):
            foo = Foo(name='blah')
        
        with self.assertRaises(AssertionError):
            with self.env.read():
                foo = Foo(name='blah')
        
        with self.env.write():
            foo = Foo(name="Snark Jones")
            foo_id = foo.id
        
        with self.env.read():
            foo = Foo.get(foo_id)
            with self.assertRaises(AssertionError):
                foo.name = "Armchair Cowboy"
            
            
    def test_iterate(self):
        class Foo(self.env.Model):
            name = Text()
        
        names = []
        with self.env.write():
            for i in range(0,10):
                name = "Foo #%s" % i
                names.append(name)
                foo = Foo(name=name)
        
        fetched_names = []
        with self.env.read():
            for f in Foo.cursor():
                fetched_names.append(f.name)
                
        self.assertEquals(fetched_names, names)
        
        
    def test_iterate_backwards(self):
        class Foo(self.env.Model):
            name = Text()
            
        names = []
        with self.env.write():
            for i in range(0,10):
                name = "Foo #%s" % i
                names.append(name)
                foo = Foo(name=name)
        
        fetched_names = []
        with self.env.read():
            for f in Foo.cursor(reverse=True):
                fetched_names.append(f.name)
                
        self.assertEquals(fetched_names, list(reversed(names)))
        
        
    def test_range(self):
        class Foo(self.env.Model):
            pass
        
        ids = []
        with self.env.write():
            for i in range(0,10):
                foo = Foo()
                ids.append(foo.id)
        
        fetched_ids = []
        with self.env.read():
            for f in Foo.cursor().range(ids[5]):
                fetched_ids.append(f.id)
        
        self.assertEquals(fetched_ids, ids[5:])
        
        
    def test_range_end(self):
        class Foo(self.env.Model):
            pass
        
        ids = []
        with self.env.write():
            for i in range(0,10):
                foo = Foo()
                ids.append(foo.id)
        
        fetched_ids = []
        with self.env.read():
            for f in Foo.cursor().range(ids[5], ids[7]):
                fetched_ids.append(f.id)
        
        self.assertEquals(fetched_ids, ids[5:7])
        
        
    def test_range_backwards(self):
        class Foo(self.env.Model):
            pass
        
        ids = []
        with self.env.write():
            for i in range(0,10):
                foo = Foo()
                ids.append(foo.id)
        
        fetched_ids = []
        with self.env.read():
            for f in Foo.cursor(reverse=True).range(ids[5], ids[7]):
                fetched_ids.append(f.id)
        
        self.assertEquals(fetched_ids, list(reversed(ids[5:7])))
        
        
    def test_count(self):
        class Foo(self.env.Model):
            pass
            
        with self.env.write():
            for i in range(0,10):
                foo = Foo()
        
        with self.env.read():
            self.assertEquals(Foo.count(), 10)
            
            
    def test_simple_inheritance(self):
        class Foo(self.env.Model):
            pass
            
        class Bar(Foo):
            pass
        
        with self.env.write():
            for i in range(0,10):
                f = Foo(id=i)
            for i in range(10,17):
                b = Bar(id=i)
        
        with self.env.read():
            self.assertEquals(Foo.count(), 17)
            self.assertEquals(Bar.count(), 7)
            
            for inst in Foo.cursor():
                self.assertIsInstance(inst, (Foo, Bar))
            
            for b in Bar.cursor():
                self.assertIsInstance(b, Bar)
                
            b = Foo.get(15)
            self.assertIsInstance(b, Bar)
            
            b = Bar.get(15)
            self.assertIsInstance(b, Bar)
            
            f = Foo.get(3)
            self.assertIsInstance(f, Foo)
                
                
    def test_inherited_indexes(self):
        class Foo(self.env.Model):
            skidoo = TypeOf(int)
            indexes = ['skidoo']
            
        class Bar(Foo):
            pass
            
        with self.env.write():
            for i in range(0,7):
                f = Foo(skidoo=i)
            for i in range(7,10):
                b = Bar(skidoo=i)
                
        with self.env.read():
            self.assertEquals(Foo.indexes['skidoo'].count(), 10)
            self.assertEquals(Bar.indexes['skidoo'].count(), 3)
            f = Foo.indexes['skidoo'].get(3)
            self.assertEquals(f.__class__.__name__, 'Foo')
            b = Foo.indexes['skidoo'].get(7)
            self.assertEquals(b.__class__.__name__, 'Bar')
            b = Bar.indexes['skidoo'].get(3)
            self.assertEquals(b, None)
            
            
    def test_multiple_descendants(self):
        class Foo(self.env.Model):
            skidoo = TypeOf(int)
            indexes = ['skidoo']
            
        class Bar(Foo):
            pass
            
        class Baz(Foo):
            pass
            
            
        with self.env.write():
            for i in range(0,5):
                f = Foo(skidoo=i)
            for i in range(10,13):
                b = Bar(skidoo=i)
            for i in range(20,27):
                b = Baz(skidoo=i)
        
        with self.env.read():
            self.assertEquals(Foo.count(), 15)
            x = Foo.indexes['skidoo'].get(3)
            self.assertEquals(x.__class__.__name__, 'Foo')
            x = Foo.indexes['skidoo'].get(11)
            self.assertEquals(x.__class__.__name__, 'Bar')
            x = Foo.indexes['skidoo'].get(23)
            self.assertEquals(x.__class__.__name__, 'Baz')
            


class AbstractDBM(object):
    
    def __init__(self, path):
        raise NotImplementedError
        
    
    def add_namespace(self, namespace, duplicate_keys=False):
        raise NotImplementedError
        
    
    def transaction_start(self, writable=False):
        raise NotImplementedError
        
        
    def transaction_commit(self):
        raise NotImplementedError
        
        
    def transaction_abort(self):
        raise NotImplementedError
        
        
    def is_transaction_writable(self):
        raise NotImplementedError
        
        
    def put(self, namespace, key, value):
        raise NotImplementedError
        
        
    def delete(self, namespace, key, value=None):
        raise NotImplementedError
        
        
    def delete_all(self, namespace):
        raise NotImplementedError
        
        
    def get(self, namespace, key):
        raise NotImplementedError
        
        
    def cursor(self, namespace):
        raise NotImplementedError
        
        
    def count(self, namespace):
        raise NotImplementedError
        
        
    def close(self):
        raise NotImplementedError
        
        
        
class AbstractDBMCursor(object):
    
    def first(self):
        raise NotImplementedError
        
        
    def last(self):
        raise NotImplementedError
        
        
    def next(self):
        raise NotImplementedError
        
        
    def prev(self):
        raise NotImplementedError
        
        
    def jump(self, key):
        raise NotImplementedError
        
        
    def key(self):
        raise NotImplementedError
        
        
    def value(self):
        raise NotImplementedError
        
        
    def iternext(self):
        raise NotImplementedError
        
        
    def iterprev(self):
        raise NotImplementedError
        


import unittest
import os.path
import shutil
from handbag import database

TEST_PATH = "/tmp/handbag-test.db"
TEST_URL = "lmdb://%s" % TEST_PATH


class TestCursor(unittest.TestCase):
    
    def setUp(self):
        if os.path.exists(TEST_PATH):
            shutil.rmtree(TEST_PATH)
        self.db = database.open(TEST_URL)
        self.table = self.db.foo
        self.records = []
        
        with self.db.write():
            for i in range(0,20):
                rec = {
                    'id': '%.2d' % i,
                    'value': "foo.%d" % i
                }
                self.table.save(rec)
                self.records.append(rec)
                
                
    def test_first(self):
        with self.db.read():
            cur = self.table.cursor()
            doc = cur.first()
            self.assertEqual(doc, {'id':'00', 'value':'foo.0'})
            
            
    def test_last(self):
        with self.db.read():
            cur = self.table.cursor()
            doc = cur.last()
            self.assertEqual(doc, {'id':'19', 'value':'foo.19'})
            
            
    def test_all(self):
        with self.db.read():
            records = list(self.table.cursor())
            self.assertEqual(records, self.records)
            
            
    def test_range(self):
        with self.db.read():
            records = list(self.table.cursor().range('03','07'))
            self.assertEqual(records, self.records[3:7])
            
            
    def test_prefix(self):
        with self.db.read():
            records = list(self.table.cursor().prefix('0'))
            self.assertEqual(records, self.records[:10])
            
            
    def test_key(self):
        with self.db.read():
            records = list(self.table.cursor().key('04'))
            self.assertEqual(records, self.records[4:5])
            
            
    def test_first_reverse(self):
        with self.db.read():
            cur = self.table.cursor(reverse=True)
            doc = cur.first()
            self.assertEqual(doc, {'id':'19', 'value':'foo.19'})
            
            
    def test_last_reverse(self):
        with self.db.read():
            cur = self.table.cursor(reverse=True)
            doc = cur.last()
            self.assertEqual(doc, {'id':'00', 'value':'foo.0'})
            
            
    def test_all_reverse(self):
        with self.db.read():
            records = list(self.table.cursor(reverse=True))
            self.assertEqual(records, list(reversed(self.records)))
            
            
    def test_range_reverse(self):
        with self.db.read():
            records = list(self.table.cursor(reverse=True).range('03','07'))
            self.assertEqual(records, list(reversed(self.records[3:7])))
            
            
    def test_prefix_reverse(self):
        with self.db.read():
            records = list(self.table.cursor(reverse=True).prefix('1'))
            self.assertEqual(records, list(reversed(self.records[10:])))
            
            
    def test_key_reverse(self):
        with self.db.read():
            records = list(self.table.cursor(reverse=True).key('04'))
            self.assertEqual(records, self.records[4:5])
            


import dson
import cursor
import uniqueid
import index


class Table(object):
    
    def __init__(self, dbm, name):
        self.dbm = dbm
        self.name = name
        self.dbm.add_namespace(name)
        self.indexes = index.IndexCollection(self.dbm, self.name)
        
        
    def save(self, doc):
        assert self.dbm.is_transaction_writable(), "Transaction is read-only"
        if 'id' in doc:
            old_doc = self.get(doc['id'])
        else:
            doc['id'] = uniqueid.create()
            old_doc = None
        key = dson.dumpone(doc['id'])
        value = dson.dumps(doc)
        self.dbm.put(self.name, key, value)
        self.indexes.update(old_doc, doc)
        return doc
        
        
    def remove(self, id):
        assert self.dbm.is_transaction_writable(), "Transaction is read-only"
        doc = self.get(id)
        self.indexes.remove(doc)
        key = dson.dumpone(id)
        self.dbm.delete(self.name, key)
        
        
    def remove_all(self):
        assert self.dbm.is_transaction_writable(), "Transaction is read-only"
        self.dbm.delete_all(self.name)
        
        
    def get(self, id):
        key = dson.dumpone(id)
        value = self.dbm.get(self.name, key)
        if value is None:
            return None
        else:
            return dson.loads(value)
            
            
    def count(self):
        return self.dbm.count(self.name)
        
            
    def cursor(self, reverse=False):
        return cursor.Cursor(self.dbm, self.name, reverse=reverse)
        


"""
This package provides a simple validation system as well as a number of 
validators and converters. You'll probably find it's usage familiar.

Validators generally perform one task: check that a value meets a certain
specification. Sometimes they also perform a conversion. For instance, a time
validator might validate that a value is in one of a number of formats and then
convert the value to, say, a datetime instance or a POSIX timestamp.

This package tries to supply a number of flexible validators but doesn't even
come close to the kitchen sink. Most applications will have their own validation 
requirements and the thinking here is to make it as easy as possible to create 
new validators.

So, creating validators is straightforward. Subclass Validator and implement 
the validate() method. That's all that's required. Here is an example that will 
only validate yummy things. Optionally, it will convert to yumminess level::

    class Yummy(Validator):
        yumminess = {
            'Pizza': 1.5,
            'Pie': 2.75,
            'Steak': 5.58,
            'Sushi': 14.62,
            'Duck Confit': 28.06
        }
        
        def __init__(self, should_convert=False, **kwargs):
            self.should_convert = should_convert
            super(Yummy, self).__init__(**kwargs)
            
            
        def validate(self, value):
            if value not in self.yumminess:
                raise InvalidError('Yumminess not known for "%s"' % value)
            
            if self.should_convert:
                return self.yumminess[value]
            
            return value

It's a convention, but not a requirement, to put error values in the class like so::

    class Yummy(Validator):
        NOT_YUMMY = "This is not yummy"
        
        def validate(self, value):
            if not self.is_yummy(value):
                raise InvalidError(self.NOT_YUMMY)

Then we can do things based on the type of error, if we so desire::

    yummy_validator = Yummy()
    try:
        yummy_validator.validate('Fried Okra')
        print "Fried Okra is yummy!"
    except InvalidError, e:
        if e.message == Yummy.NOT_YUMMY:
            print "Fried Okra is not yummy"
        else:
            print "There is something wrong with Fried Okra"

"""

import re
from datetime import datetime

__all__ = [
    'InvalidError',
    'InvalidGroupError',
    'Validator',
    'GroupValidator',
    'Text',
    'Email',
    'DateTime',
    'Bool',
    'BoundingBox',
    'LatLng',
    'Enum',
    'TypeOf',
    'URL',
    'OneOf',
    'ListOf',
    'Anything'
]

class InvalidError(Exception):
    """
    This exception is thrown by all validators to indicate that data is invalid::
    
        v = SomeValidator()
        try:
            v.validate("puppies")
        except InvalidError, e:
            print "Oh no! Something's wrong! %s" % e.message
    """


class InvalidGroupError(InvalidError):
    """
    This exception represents a group of invalid errors. It takes a dict where
    the keys are, presumably, the names of items that were invalid and the values
    are the errors for their respective items::
        
        
        def check_stuff_out(stuff):
            validator = SomeValidator()
            errors = {}
            
            for k,v in stuff.items():
                try:
                    validator.validate(v)
                except InvalidError, e:
                    errors[k] = e
            
            if errors:
                raise InvalidGroupError(errors)
    """
    
    def __init__(self, errors):
        self.errors = errors
        super(InvalidGroupError, self).__init__(
            "Some errors were encountered\n" + \
            '\n'.join(["%s: %s" % (k,v) for k,v in errors.items()]))



class Validator(object):
    """
    This is the base validator class. Don't use it unless you are subclassing to
    create your own validator.
    """
    
    def __init__(self, optional=False, default=None, index=False):
        self._optional = optional
        self._default = default
        self._index = index
    
    def validate(self, value):
        raise NotImplementedError
        
    def default(self):
        return self._default
        
    def is_indexable(self):
        return self._index


class GroupValidator(Validator):
    """
    Validates a a dict of `key => validator`::
    
        v = GroupValidator(foo=Text(), bar=TypeOf(int))
        v.validate({'foo':'oof', 'bar':23}) # ok
        v.validate(5) # nope
        v.validate('foo':'gobot', 'bar':'pizza') # nope
        
        # unspecified keys are filtered
        v.validate({'foo': 'a', 'goo': 'b', 'bar':17})
        # ok -> {'foo': 'a', 'bar':17}
        
        # optional validators are, well, optional
        v = GroupValidator(foo=Text(), bar=TypeOf(int, optional=True, default=8))
        v.validate({'foo':'ice cream'}) # -> {'foo':'ice cream', 'bar': 8}
    """
    NOT_A_DICT = "Not a dict"
    MISSING_REQUIRED = "This field is required."
    
    def __init__(self, **kwargs):
        keys = ['optional', 'default']
        newkwargs = {}
        
        for k in keys:
            if k in kwargs:
                newkwargs[k] = kwargs.pop(k)
        
        Validator.__init__(self, **newkwargs)
        self.validators = kwargs
        
        
    def validate(self, value):
        if not isinstance(value, dict):
            raise InvalidError(self.NOT_A_DICT)
        
        validated = {}
        errors = {}
        
        for k,v in self.validators.items():
            if k in value and value[k] != "" and value[k] is not None:
                try:
                    validated[k] = v.validate(value[k])
                except InvalidError, e:
                    errors[k] = e
            else:
                if not v._optional:
                    errors[k] = self.MISSING_REQUIRED
                    continue
                validated[k] = v._default
        
        if errors:
            raise InvalidGroupError(errors)
        
        return validated


class Text(Validator):
    """
    Passes text, optionally checking length::
    
        v = Text(minlength=2,maxlength=7)
        v.validate("foo") # ok
        v.validate(23) # oops
        v.validate("apron hats") # oops
        v.validate("f") # oops
    """
    NOT_TEXT = 'Expected some text.'
    TOO_SHORT = 'This text is too short.'
    TOO_LONG = 'This text is too long.'
    
    def __init__(self, minlength=None, maxlength=None, **kwargs):
        self.minlength = minlength
        self.maxlength = maxlength
        super(Text, self).__init__(**kwargs)
        
    def validate(self, value):
        if not isinstance(value, basestring):
            raise InvalidError(self.NOT_TEXT)
            
        if self.minlength is not None and len(value) < self.minlength:
            raise InvalidError(self.TOO_SHORT)
            
        if self.maxlength is not None and len(value) > self.maxlength:
            raise InvalidError(self.TOO_LONG)
            
        return value


class Email(Validator):
    """
    Passes email addresses that meet the guidelines in `RFC 3696 <http://tools.ietf.org/html/rfc3696>`_::
    
        v = Email()
        v.validate("foo@example.com") # ok
        v.validate("foo.bar_^&!baz@example.com") # ok
        v.validate("@example.com") # oops!
    """
    NOT_EMAIL = "Invalid email address"
    pattern = re.compile("^((\".+\")|((\\\.))|([\d\w\!#\$%&'\*\+\-/=\?\^_`\{\|\}~]))((\"[^@]+\")|(\\\.)|([\d\w\!#\$%&'\*\+\-/=\?\^_`\.\{\|\}~]))*@[a-zA-Z0-9]+([a-zA-Z0-9\-][a-zA-Z0-9]+)?(\.[a-zA-Z0-9]+([a-zA-Z0-9\-][a-zA-Z0-9]+)?)+\.?$")
    
    def validate(self, value):
        if not isinstance(value, basestring):
            raise InvalidError(self.NOT_EMAIL)
        if not self.pattern.match(value):
            raise InvalidError(self.NOT_EMAIL)
        return value


try:
    import timelib
    strtodatetime = timelib.strtodatetime
except ImportError:
    strtodatetime = None

try:
    from dateutil import parser as date_parser
    parsedate = date_parser.parse
except ImportError:
    parsedate = None

class DateTime(Validator):
    """
    Validates many representations of date & time and converts to datetime.datetime.
    It will use `timelib <http://pypi.python.org/pypi/timelib/>`_ if available,
    next it will try `dateutil.parser <http://labix.org/python-dateutil>`_. If neither
    is found, it will use :func:`datetime.strptime` with some predefined format string.
    Int or float timestamps will also be accepted and converted::
    
        # assuming we have timelib
        v = DateTime()
        v.validate("today") # datetime.datetime(2011, 9, 17, 0, 0, 0)
        v.validate("12:06am") # datetime.datetime(2011, 9, 17, 0, 6)
        v.validate(datetime.now()) # datetime.datetime(2011, 9, 17, 0, 7)
        v.validate(1316232496.342259) # datetime.datetime(2011, 9, 17, 4, 8, 16, 342259)
        v.validate("baloon torches") # oops!
    """
    NOT_DATE = "Unrecognized date format"
    
    def __init__(self, default_format="%x %X", use_timelib=True, 
            use_dateutil=True, **kwargs):
        super(DateTime, self).__init__(**kwargs)
        self._default_format = default_format
        self._use_timelib = use_timelib
        self._use_dateutil = use_dateutil
        
        
    def validate(self, value):
        if isinstance(value, datetime):
            return value
        
        if isinstance(value, int) or isinstance(value, float):
            try:
                return datetime.utcfromtimestamp(value)
            except:
                raise InvalidError(self.NOT_DATE)
        
        if not isinstance(value, basestring):
            raise InvalidError, "Note a date or time"
        
        if self._use_timelib and strtodatetime:
            try:
                return strtodatetime(value)
            except:
                raise InvalidError(self.NOT_DATE)
        
        if self._use_dateutil and parsedate:
            try:
                return parsedate(value)
            except:
                raise InvalidError(self.NOT_DATE)
        
        try:
            return datetime.strptime(value, self._default_format)
        except:
            raise InvalidError(self.NOT_DATE)
            
            
class Bool(Validator):
    """
    Passes and converts most representations of True and False::
        
        b = Bool()
        b.validate("true") # True
        b.validate(1) # True
        b.validate("yes") # True
        b.validate(True) # True
        b.validate("false") # False, etc.
        
    """
    NOT_BOOL = "Not a boolean"
    
    
    def validate(self, value):
        if isinstance(value, basestring):
            v = value.lower()
            if v in ["true", "1", "yes"]:
                return True
            elif v in ["false", "0", "no"]:
                return False
            else:
                raise InvalidError(self.NOT_BOOL)
        elif isinstance(value, int):
            if value == 1:
                return True
            elif value == 0:
                return False
            else:
                raise InvalidError(self.NOT_BOOL)
        elif isinstance(value, bool):
            return value
        raise InvalidError(self.NOT_BOOL)
        
        
class BoundingBox(Validator):
    """
    Passes a geographical bounding box of the form SWNE, e.g., 37.73,-122.48,37.78,-122.37.
    It will accept a list or a comma-separated string::
    
        v = BoundingBox()
        b.validate([42.75804,-85.0031, 42.76409, -84.9861]) # ok
        b.validate("42.75804,-85.0031, 42.76409, -84.9861") # -> [42.75804,-85.0031, 42.76409, -84.9861]
    """
    VALUES_OUT_OF_RANGE = "All values must be numbers in the range -180.0 to 180.0"
    WRONG_SIZE = "A bounding box must have 4 values"
    NOT_STRING_OR_LIST = "Expected a comma-separated list of values or a list or tuple object."
    
    def validate(self, value):
        if not isinstance(value, (list, tuple)):
            if isinstance(value, basestring):
                value = [v.strip() for v in value.split(',')]
            else:
                raise InvalidError(self.NOT_STRING_OR_LIST)
        
        if len(value) != 4:
            raise InvalidError(self.WRONG_SIZE)
            
        try:
            value = [float(v) for v in value]
        except ValueError:
            raise InvalidError(self.VALUES_OUT_OF_RANGE)
        
        for v in value:
            if not (-180.0 <= v <= 180.0):
                raise InvalidError(self.VALUES_OUT_OF_RANGE)
        
        return tuple(value)
        
        
class LatLng(Validator):
    """
    Passes a geographical point in for form of a list, tuple or comma-separated string::
    
        v = LatLng()
        v.validate("42.76066, -84.9929") # ok -> (42.76066, -84.9929)
        v.validate((42.76066, -84.9929)) # ok
        v.validate("234,56756.453") # oops
    """
    VALUES_OUT_OF_RANGE = "All values must be numbers in the range -180.0 to 180.0"
    WRONG_SIZE = "A point must have 2 values"
    NOT_STRING_OR_LIST = "Expected a comma-separated list of values or a list or tuple object."
    
    def validate(self, value):
        if not isinstance(value, (list, tuple)):
            if not isinstance(value, basestring):
                raise InvalidError(self.NOT_STRING_OR_LIST)
            value = [v.strip() for v in value.split(",")]
        
        if len(value) != 2:
            raise InvalidError(self.WRONG_SIZE)
        
        try:
            value = [float(v) for v in value]
        except ValueError:
            raise InvalidError(self.VALUES_OUT_OF_RANGE)
        
        for v in value:
            if not (-180.0 <= v <= 180.0):
                raise InvalidError(self.VALUES_OUT_OF_RANGE)
        
        return tuple(value)
        
        
class Enum(Validator):
    """
    Passes anything that evaluates equal to one of a list of values::
    
        v = Enum('a', 'b', 'c')
        v.validate('a') # ok
        v.validate('d') # nope!
    """
    NOT_IN_LIST = "Not in the list"
    
    def __init__(self, *values, **kwargs):
        super(Enum, self).__init__(**kwargs)
        self.values = values
    
    
    def validate(self, value):
        if value in self.values:
            return value
        raise InvalidError(self.NOT_IN_LIST)
        
        
class TypeOf(Validator):
    """
    Passes any value of a specified type::
    
        v = TypeOf(float)
        v.validate(0.4) # ok
        v.validate(1) # nope
        
        # more than one type is ok too
        v = TypeOf(int, float, complex)
        v.validate(5) # ok
        v.validate(5.5) # ok
        v.validate(complex(5,5)) # ok
    """
    
    def __init__(self, *types, **kwargs):
        super(TypeOf, self).__init__(**kwargs)
        self.types = types
        
        
    def validate(self, value):
        if not isinstance(value, self.types):
            raise InvalidError, "Not of type '%s'" % (self.types,)
        return value
        
        
class URL(Validator):
    """
    Passes a URL using guidelines from RFC 3696::
        
        v = URL()
        v.validate('http://www.example.com') # ok
        v.validate('https://www.example.com:8000/foo/bar?smelly_ones=true#dsfg') # ok
        v.validate('http://www.example.com/foo;foo') # nope
        
        # You can also set which schemes to match
        v = URL(schemes=('gopher',))
        v.validate('gopher://example.com/') # ok
        v.validate('http://example.com/') # nope!
    
    Regex from http://daringfireball.net/2010/07/improved_regex_for_matching_urls.
    """
    NOT_A_URL = "Not a URL"
    
    def __init__(self, **kwargs):
        super(URL, self).__init__(**kwargs)
        self.pattern = re.compile('((?:[a-z][\w-]+:(?:/{1,3}|[a-z0-9%])|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:\'".,<>?\xc2\xab\xc2\xbb\xe2\x80\x9c\xe2\x80\x9d\xe2\x80\x98\xe2\x80\x99]))')
        
        
    def validate(self, value):
        if not isinstance(value, basestring):
            raise InvalidError(self.NOT_A_URL)
        if not self.pattern.match(value):
            raise InvalidError(self.NOT_A_URL)
        return value
        
        
class OneOf(Validator):
    """
    Passes values that pass one of a list of validators::
    
        v = OneOf(URL(), Enum('a', 'b'))
        v.validate('b') # ok
        v.validate('http://www.example.com') # ok
        v.validate(23) # nope
    """
    
    def __init__(self, *validators, **kwargs):
        super(OneOf, self).__init__(**kwargs)
        self.validators = validators
        
        
    def validate(self, value):
        is_valid = False
        
        for v in self.validators:
            try:
                v.validate(value)
                is_valid = True
                break
            except InvalidError:
                pass
        
        if is_valid:
            return value
            
        raise InvalidError("Didn't match any validators")
        
        
class ListOf(Validator):
    """
    Passes a list of values that pass a validator::
    
        v = ListOf(TypeOf(int))
        v.validate([1,2,3]) # ok
        v.validate([1,2,"3"]) # nope
        v.validate(1) # nope
    """
    NOT_A_LIST = "Not a list"
    
    def __init__(self, validator, **kwargs):
        super(ListOf, self).__init__(**kwargs)
        self.validator = validator
        
        
    def validate(self, values):
        if not isinstance(values, list):
            raise InvalidError(self.NOT_A_LIST)
        
        for v in values:
            self.validator.validate(v)
        
        return values
        
        
class Anything(Validator):
    """
    Passes anything
    """
    
    def validate(self, value):
        return value



"""
Unit tests for data validators
"""
import unittest
from handbag.validators import *
from datetime import datetime
import time


class TestAbstractValidator(unittest.TestCase):
    
    def test_validator_abstract(self):
        """
        Shouldn't be able to use base Validator as a validator
        """
        v = Validator()
        self.assertRaises(NotImplementedError, v.validate, 23)
        
        
class TestText(unittest.TestCase):
        
    def test_not_text(self):
        """
        Should raise InvalidError for anything but a string
        """
        v = Text()
        self.assertRaises(InvalidError, v.validate, 23)
        
        
    def test_text(self):
        """
        Should not raise InvalidError for text
        """
        v = Text()
        try:
            v.validate("foo")
        except InvalidError:
            self.fail("Text.validate() raised InvalidError for a string")
            
            
    def test_minlength_success(self):
        """
        Should not raise InvalidError if a string is longer than minlength.
        """
        test_string = "foofoo"
        v = Text(minlength=len(test_string)-1)
        
        try:
            v.validate(test_string)
        except InvalidError:
            self.fail("Text.validate(minlength=%d) raised InvalidError for a string of length %d" %(
                len(test_string)-1, len(test_string)))
            
            
    def test_minlength_fail(self):
        """
        Should raise InvalidError if a string is shorter than minlength.
        """
        test_string = "foofoo"
        v = Text(minlength=len(test_string)+1)
        self.assertRaises(InvalidError, v.validate, test_string)
        
        
    def test_maxlength_success(self):
        """
        Should not raise InvalidError if a string's length is less than or equal to maxlength
        """
        test_string = "foofoo"
        test_string_len = len(test_string)
        v = Text(maxlength=test_string_len)
        
        try:
            v.validate(test_string)
        except InvalidError:
            self.fail("Text.validate(maxlength=%d) raised InvalidError for a string of length %d" %(
                test_string_len, test_string_len))
            
            
        v = Text(maxlength=test_string_len+1)
        
        try:
            v.validate(test_string)
        except InvalidError:
            self.fail("Text.validate(maxlength=%d) raised InvalidError for a string of length %d" %(
                test_string_len+1, test_string_len))
            
            
    def test_maxlength_fail(self):
        """
        Should raise InvalidError if a string is longer than maxlength
        """
        test_string = "foofoo"
        v = Text(maxlength=len(test_string)-1)
        self.assertRaises(InvalidError, v.validate, test_string)
        
        
    def test_return_value(self):
        """
        Should always return the input value
        """
        string = "foobarbaz"
        v = Text(minlength=1,maxlength=23)
        self.assertEqual(string, v.validate(string))
        
        
class TestEmail(unittest.TestCase):
    
    def test_fail(self):
        """
        Should raise InvalidError on things that aren't email addresses
        """
        v = Email()
        not_emails = [
            23,
            "foo",
            "@foo.com"
        ]
        
        for not_email in not_emails:
            self.assertRaises(InvalidError, v.validate, not_email)
        
        
    def test_pass(self):
        """
        Should accept addresses that meet the guidelines in RFC 3696
        """
        emails = [
            'foo@example.com',
            'a@b.c',
            'foo@exa.amp.le.com',
            'foo_bar@example.com.',
            'foo.bar@example.com',
            'f!o#o$b%a&r\'b*a+z-b/a=n?g^b_i`f.b{o|p}~@example.com',
            '"some white space"@example.com',
            '\@fo"   "\//o@example.com'
        ]
        
        v = Email()
        
        for email in emails:
            try:
                self.assertEquals(email, v.validate(email))
            except InvalidError:
                self.fail('Failed to accept %s' % email)
        
        
class TestDateTime(unittest.TestCase):
    
    def test_fail(self):
        """
        Should raise invalid for things that aren't dates and times
        """
        v = DateTime()
        not_datetimes = [
            "santa claus",
            "the cure is wednesday",
            "324"
        ]
        
        for not_datetime in not_datetimes:
            self.assertRaises(InvalidError, v.validate, not_datetime)
        
        
    def test_simple(self):
        """
        Should parse dates in the default format and return them as datetime objects.
        """
        v = DateTime(default_format="%x")
        bday = datetime(1982, 9, 6)
        
        try:
            self.assertEqual(bday, v.validate('9/6/82'))
        except InvalidError:
            self.fail('Failed to validate 9/6/82 with format %x')
        
        
    def test_timelib(self):
        """
        If timelib is installed, should be able to parse stuff like "today"
        """
        try:
            import timelib
        except ImportError:
            print "timelib is not installed, skipping timelib test for DateTime validator"
            return
        
        v = DateTime(use_timelib=True, use_dateutil=False)
        today = datetime.utcnow().replace(hour=0,minute=0,second=0,microsecond=0)
        
        self.assertEqual(today, v.validate("today"))
        
        
    def test_dateutil(self):
        """
        If dateutil is installed, should be able to parse formats other than the default
        """
        try:
            import dateutil
        except ImportError:
            print "dateutil is not installed, skipping dateutil test for DateTime validator"
            return
        
        v = DateTime(default_format="%d", use_timelib=False, use_dateutil=True)
        bday = datetime(1982,9,6)
        
        self.assertEqual(bday, v.validate('9/6/82'))
            
            
    def test_reflexive(self):
        """
        Should pass and return a datetime.datetime instance.
        """
        v = DateTime()
        now = datetime.now()
        
        try:
            self.assertEquals(now, v.validate(now))
        except InvalidError:
            self.fail("Failed to pass a datetime.datetime instance")
            
            
    def test_timestamp(self):
        """
        Should pass both int and float timestamps and convert to utc datetime
        """
        v = DateTime()
        now = time.time()
        now_date = datetime.utcfromtimestamp(now)
        
        try:
            self.assertEquals(now_date, v.validate(now))
        except InvalidError:
            self.fail("Failed to pass a float timestamp")
            
        now = int(now)
        now_date = datetime.utcfromtimestamp(now)
        
        try:
            self.assertEquals(now_date, v.validate(now))
        except InvalidError:
            self.fail("Failed to pass an int timestamp")
        
        
class TestBool(unittest.TestCase):
    
    def test_fail(self):
        """
        Should raise InvalidError for things that don't represent booleans
        """
        not_bools = [
            "maybe",
            67,
            dict
        ]
        
        v = Bool()
        
        for not_bool in not_bools:
            self.assertRaises(InvalidError, v.validate, not_bool)
        
        
    def test_falses(self):
        """
        Should pass and convert several represenations of False
        """
        falses = [
            False,
            0,
            "0",
            "false",
            "FALSE",
            "fAlSe",
            "no",
            "No"
        ]
        
        v = Bool()
        
        for false in falses:
            try:
                self.assertEqual(False, v.validate(false))
            except InvalidError:
                self.fail("Didn't pass '%s'" % false)
        
        
    def test_falses(self):
        """
        Should pass and convert several represenations of True
        """
        trues = [
            True,
            1,
            "1",
            "true",
            "TRUE",
            "TruE",
            "yes",
            "yEs"
        ]
        
        v = Bool()
        
        for true in trues:
            try:
                self.assertEqual(True, v.validate(true))
            except InvalidError:
                self.fail("Didn't pass '%s'" % true)
        
        
        
class TestBoundingBox(unittest.TestCase):
    
    def test_fail(self):
        """
        Shouldn't pass things that aren't a bounding box.
        """
        not_bboxes = [
            "fred",
            (1,3,4),
            ["apples", 42, 24, 6],
            (1,181,3,-300),
            348.345
        ]
        v = BoundingBox()
        
        for not_bbox in not_bboxes:
            try:
                v.validate(not_bbox)
                self.fail("Passed '%s'" % (not_bbox,))
            except InvalidError:
                pass
        
        
    def test_list(self):
        """
        Should pass a list or tuple
        """
        box = [42.75804,-85.0031, 42.76409, -84.9861]
        v = BoundingBox()
        
        try:
            v.validate(box)
        except InvalidError:
            self.fail("Failed '%s'", box)
            
        box = tuple(box)
        
        try:
            v.validate(box)
        except InvalidError:
            self.fail("Failed '%s'", box)
        
        
    def test_string(self):
        """
        Should pass a comma-separated string and convert to a tuple
        """
        box = (42.75804,-85.0031, 42.76409, -84.9861)
        str_box = ",".join([str(b) for b in box])
        v = BoundingBox()
        
        try:
            self.assertEqual(box, v.validate(str_box))
        except InvalidError:
            self.fail("Failed '%s'", str_box)
        
        
class TestLatLng(unittest.TestCase):
    
    def test_fail(self):
        """
        Should throw InvalidError for things that aren't a geographic point
        """
        not_latlngs = [
            14,
            "the whole earf",
            {'foo': 'bar'},
            (181,-181)
        ]
        
        v = LatLng()
        
        for not_latlng in not_latlngs:
            try:
                v.validate(not_latlng)
                self.fail("Passed %s" % (not_latlng,))
            except InvalidError:
                pass
            
            
    def test_pass(self):
        """
        Should pass a list, tuple or comma-separated string.
        """
        latlngs = [
            "42.76066, -84.9929",
            (42.76066, -84.9929),
            [42.76066, -84.9929]
        ]
        
        v = LatLng()
        
        for latlng in latlngs:
            try:
                v.validate(latlng)
            except InvalidError:
                self.fail("Failed to pass %s" % latlng)
        
        
    def test_return_value(self):
        """
        Should always return a 2-tuple of floats
        """
        latlng = (42.76066, -84.9929)
        latlngs = [list(latlng), "%s,%s" % latlng]
        
        v = LatLng()
        
        for l in latlngs:
            self.assertEqual(latlng, v.validate(l))
        
        
class TestEnum(unittest.TestCase):
    
    def test_fail(self):
        """
        Shouldn't pass anything not in the defined list
        """
        values = [5, "atilla the hun", unicode]
        wrong = [8, "ivan the terrible", str]
        
        v = Enum(*values)
        
        for w in wrong:
            self.assertRaises(InvalidError, v.validate, w)
            
            
    def test_pass(self):
        """
        Should pass anything in the defined list
        """
        values = [5, "atilla the hun", unicode]
        
        v = Enum(*values)
        
        for value in values:
            try:
                self.assertEqual(value, v.validate(value))
            except InvalidError:
                self.fail("Didn't pass %s" % value)
        
        
class TestTypeOf(unittest.TestCase):
    
    def test_fail(self):
        """
        Shouldn't pass values of a type not specified
        """
        v = TypeOf(int)
        self.assertRaises(InvalidError, v.validate, "Hi, hungry?")
        
        
    def test_pass(self):
        """
        Should pass values of the specified type
        """
        v = TypeOf(basestring)
        value = u"foo"
        
        try:
            self.assertEqual(value, v.validate(value))
        except InvalidError:
            self.fail("Didn't pass value of specified type")
        
        
class TestURL(unittest.TestCase):
    
    def test_fail(self):
        """
        Don't pass things that aren't URLs.
        """
        not_urls = ["snipe", u'\xe2\x99\xa5', 777]
        v = URL()
        
        for not_url in not_urls:
            self.assertRaises(InvalidError, v.validate, not_url)
        
        
    def test_pass(self):
        """
        Should pass URLs with the specified schemes.
        """
        urls = [
            'http://example.com',
            'foo://example.com./',
            'http://example.com/foo/bar?baz=goo&snoo=snazz#help',
            'http://127.0.0.1',
            'bar://127.0.0.1:80',
            'http://foo:123/',
            'http://foo:bar@baz:123'
        ]
        v = URL()
        
        for url in urls:
            try:
                self.assertEqual(url, v.validate(url))
            except InvalidError:
                self.fail("Didn't pass '%s'" % url)
        
        
class TestOneOf(unittest.TestCase):
    
    def test_fail(self):
        """
        Should not pass things that don't pass at least one validator.
        """
        bads = [
            "snooze button",
            50
        ]
        v = OneOf(Email(), TypeOf(float))
        
        for bad in bads:
            self.assertRaises(InvalidError, v.validate, bad)
            
            
    def test_pass(self):
        """
        Should pass anything that matches any of the validators
        """
        goods = [23, 3.1459, "batman", 16]
        v = OneOf(TypeOf(int), Enum(3.1459, "pie", "batman"))
        
        for good in goods:
            try:
                self.assertEqual(good, v.validate(good))
            except InvalidError:
                self.fail("Failed to pass '%s'" % good)
        
        
class TestListOf(unittest.TestCase):
    
    def test_fail(self):
        """
        Shouldn't pass a list of things that don't pass the validator or things
        that aren't lists.
        """
        bads = [23, [23,24,25]]
        v = ListOf(TypeOf(basestring))
        
        for bad in bads:
            self.assertRaises(InvalidError, v.validate, bad)
            
            
    def test_pass(self):
        """
        Should pass a list of things that pass the validator
        """
        goods = [
            ['a', 15, 'pointy hat'],
            [5],
            ['pancakes', 'alpha centauri', 9]
        ]
        
        v = ListOf(TypeOf(basestring, int))
        
        for good in goods:
            try:
                self.assertEqual(good, v.validate(good))
            except InvalidError:
                self.fail("Failed to pass '%s'", good)
        
        
class TestGroup(unittest.TestCase):
    
    def test_fail(self):
        """
        Shouldn't pass anything that isn't a dict with specified keys and
        validated values.
        """
        bads = [
            {
                5: "6",
                7: "8"
            },
            {
                "hoodo": Text(),
                "we": "appreciate" 
            }
        ]
        
        v = GroupValidator(
            do = Text(),
            we = Text()
        )
        
        for bad in bads:
            self.assertRaises(InvalidGroupError, v.validate, bad)
            
        self.assertRaises(InvalidError, v.validate, "not a dict")
        
        
    def test_pass(self):
        """
        Should pass dicts with specified keys and validated values. Should 
        also return a dict with converted values.
        """
        goods = [
            ({'foo':'bar', 'baz':23}, {'foo':'bar', 'baz':23}),
            ({'foo':'bar'}, {'foo':'bar', 'baz':5.5})
        ]
        
        v = GroupValidator(foo=Text(), baz=TypeOf(int, float, optional=True, default=5.5))
        
        for good_in, good_out in goods:
            try:
                self.assertEqual(good_out, v.validate(good_in))
            except InvalidError:
                self.fail("Failed to pass %s" % good_in)
                
                
    def test_empty_string_optional(self):
        """
        Should not raise an error when passing an empty string to an optional validator
        """
        v = GroupValidator(email=Email(optional=True))
        try:
            v.validate({'email':''})
        except InvalidError:
            self.fail("Raised invalid error on an empty string for an optional validator")
        
        
class TestAnything(unittest.TestCase):
    
    def test_pass(self):
        """
        Should pass anything
        """
        v = Anything()
        anythings = ["foo", 7, Anything(), unittest]
        
        for anything in anythings:
            try:
                self.assertEqual(anything, v.validate(anything))
            except InvalidError:
                self.fail('Failed something')
        


import dson


class Cursor(object):
    
    def __init__(self, dbm, name, reverse=False):
        self.dbm = dbm
        self.name = name
        self.reverse = reverse
        
        
    def first(self):
        cursor = self._create_cursor()
        if self.reverse:
            if cursor.last():
                return self.load(cursor.value())
        elif cursor.first():
            return self.load(cursor.value())
        
        
    def last(self):
        cursor = self._create_cursor()
        if self.reverse:
            if cursor.first():
                return self.load(cursor.value())
        elif cursor.last():
            return self.load(cursor.value())
        
        
    def __iter__(self):
        return self.get_iterator('all')
        
        
    def range(self, start=None, end=None):
        if start is None and end is None:
            return self.__iter__()
            
        return self.get_iterator(
            'range',
            start if start is None else self.dump_key(start),
            end if end is None else self.dump_key(end)
        )
        
        
    def prefix(self, prefix):
        return self.get_iterator('match_prefix', self.dump_prefix(prefix))
        
        
    def key(self, key):
        return self.get_iterator('match_key', self.dump_key(key))
        
        
    def count_range(self, start=None, end=None):
        return self.get_count_with_iterator(
            'range',
            start if start is None else self.dump_key(start),
            end if end is None else self.dump_key(end)
        )
        
        
    def count_prefix(self, prefix):
        return self.get_count_with_iterator('match_prefix', self.dump_prefix(prefix))
        
        
    def count_key(self, key):
        return self.get_count_with_iterator('match_key', self.dump_key(key))
        
        
    def dump_key(self, key):
        return dson.dumpone(key)
        
        
    def dump_prefix(self, prefix):
        return self.dump_key(prefix)
        
        
    def load(self, data):
        return dson.loads(data)
        
        
    def get_iterator(self, name, *args):
        forward, backward = iterators.get(name)
        iterator = backward if self.reverse else forward
        for k,v in iterator(self._create_cursor(), *args):
            yield self.load(v)
            
            
    def get_count_with_iterator(self, name, *args):
        forward, backward = iterators.get(name)
        iterator = backward if self.reverse else forward
        c = 0
        for r in iterator(self._create_cursor(), *args):
            c += 1
        return c
        
        
    def _create_cursor(self):
        return self.dbm.cursor(self.name)
   
        

def iter_all(cursor):
    cursor.first()
    return cursor.iternext()
    
    
def iter_all_reverse(cursor):
    cursor.last()
    return cursor.iterprev()
    
    
def iter_range(cursor, start, end):
    if start is None:
        cursor.first()
    else:
        cursor.jump(start)
    if end:
        return iter_while(
            cursor.iternext(),
            lambda x: x < end
        )
    else:
        return cursor.iternext()
        
        
def iter_range_reverse(cursor, start, end):
    if end is None:
        cursor.last()
    else:
        cursor.jump(end)
        cursor.prev()
    if start:
        return iter_while(
            cursor.iterprev(), 
            lambda x: x >= start
        )
    else:
        return cursor.iterprev()
        
        
def iter_match_prefix(cursor, prefix):
    cursor.jump(prefix)
    return iter_while(
        cursor.iternext(), 
        lambda x: x.startswith(prefix)
    )
    
    
def iter_match_prefix_reverse(cursor, prefix):
    last_char = ord(prefix[-1])
    if last_char < 255:
        prefix_bound = prefix[:-1] + chr(last_char + 1)
    else:
        prefix_bound = prefix + '\x00'
        
    cursor.jump(prefix_bound)
    cursor.prev()
    
    return iter_while(
        cursor.iterprev(), 
        lambda x: x.startswith(prefix)
    )
    
    
def iter_match_key(cursor, key):
    cursor.jump(key)
    return iter_while(
        cursor.iternext(), 
        lambda x: x == key
    )
    
    
def iter_match_key_reverse(cursor, key):
    key_bound = key + '\x00'
    cursor.jump(key_bound)
    cursor.prev()
    return iter_while(
        cursor.iterprev(),
        lambda x: x == key
    )
        
        
def iter_while(iterator, predicate):
    for key, value in iterator:
        if predicate(key):
            yield key, value
        else:
            break
            
            
iterators = {
    'all': (iter_all, iter_all_reverse),
    'range': (iter_range, iter_range_reverse),
    'match_prefix': (iter_match_prefix, iter_match_prefix_reverse),
    'match_key': (iter_match_key, iter_match_key_reverse)
}



from setuptools import setup, find_packages
execfile('handbag/version.py')

setup(
    name = 'handbag',
    version = __version__,
    packages = find_packages(),
    description = 'An embedded database and data modeling library for python.',
    author = 'Elisha Fitch-Cook',
    author_email = 'elisha@elishacook.com',
    url = 'https://github.com/elishacook/handbag',
    test_suite = 'tests',
    install_requires=[
        'lmdb>=0.78',
        'pytz'
    ]
)



import unittest
import os.path
import shutil
from handbag import environment
from handbag.validators import *
from handbag.relationships import *

TEST_PATH = "/tmp/handbag-test.db"
TEST_URL = "lmdb://%s" % TEST_PATH


class TestRelationships(unittest.TestCase):
    
    def setUp(self):
        if os.path.exists(TEST_PATH):
            shutil.rmtree(TEST_PATH)
        self.env = environment.open(TEST_URL)
    
    
    def test_one(self):
        class Foo(self.env.Model):
            bar = One("Bar")
            
        class Bar(self.env.Model):
            pass
        
        with self.env.write():
            foo = Foo()
            bar = Bar()
            foo.bar = bar
            self.assertEquals(foo.bar, bar)
            
        with self.env.read():
            foo = Foo.get(foo.id)
            self.assertEquals(foo.bar, bar)
        
      
    def test_one_no_inverse(self):
        with self.assertRaises(TypeError):
            class Foo(self.env.Model):
                bar = One("Bar", inverse="foo")
                
    
    def test_one_to_one(self):
        class Foo(self.env.Model):
            bar = OneToOne("Bar", inverse="foo")
            
        class Bar(self.env.Model):
            pass
        
        with self.env.write():
            bar = Bar()
            foo = Foo()
            foo.bar = bar
        
        with self.env.read():
            self.assertEquals(bar.foo, foo)
        
        
    def test_one_to_one_overdefined(self):
        class Foo(self.env.Model):
            bar = OneToOne("Bar", inverse="foo")
        
        class Bar(self.env.Model):
            foo = OneToOne(Foo, inverse="bar")
        
        with self.env.write():
            bar = Bar()
            foo = Foo()
            foo.bar = bar
        
        with self.env.read():
            self.assertEquals(bar.foo, foo)
        

    def test_one_to_one_cascade(self):
        class Foo(self.env.Model):
            bar = OneToOne("Bar", inverse="foo", cascade=True)
        
        class Bar(self.env.Model):
            pass
        
        with self.env.write():
            bar = Bar()
            bar2 = Bar()
            foo = Foo()
            foo.bar = bar
        
        with self.env.read():
            self.assertEquals(Bar.count(), 2)
            
        with self.env.write():
            foo.remove()
            
        with self.env.read():
            self.assertEquals(Bar.count(), 1)
            b = Bar.cursor().first()
            self.assertEquals(b, bar2)
        
        
    def test_one_to_many(self):
        data = {
            'San Francisco': [
                'Alchemist', 'Bar Tartine', 'Lounge 3411', 'Tempest'
            ],
            'New York': [
                'Dead Rabbit', 'Death & Co.', 'Donna', 'Proletariat'
            ]
        }
        
        class City(self.env.Model):
            name = Text()
            bars = OneToMany("Bar", inverse="city", indexes=['name'])
            indexes = ['name']
        
        class Bar(self.env.Model):
            name = Text()
            indexes = ['name']
        
        with self.env.write():
            for city, bars in data.items():
                c = City(name=city)
                for bar in bars:
                    b = Bar(name=bar)
                    c.bars.add(b)
        
        with self.env.read():
            city = City.indexes['name'].get('San Francisco')
            self.assertEquals(city.bars.count(), 4)
            self.assertEquals([b.name for b in city.bars.indexes['name'].cursor()], data['San Francisco'])
            
            city = City.indexes['name'].get('New York')
            self.assertEquals(city.bars.count(), 4)
            self.assertEquals([b.name for b in city.bars.indexes['name'].cursor()], data['New York'])
            
            bar = Bar.indexes['name'].get('Donna')
            self.assertEquals(bar.city.name, 'New York')
            self.assertEquals(bar.city, city)
        
        with self.env.write():
            bar.remove()
            
        with self.env.read():
            self.assertEquals(city.bars.count(), 3)
            
            b = city.bars.first()
            name = b.name
            self.assertIn(name, data['New York'])
            
        with self.env.write():
            city.bars.remove(b)
            
        with self.env.read():
            self.assertEquals(city.bars.count(), 2)
            new_b = Bar.indexes['name'].get(name)
            self.assertEquals(b, new_b)
        
        
    def test_one_to_many_overdefined(self):
        data = {
            'San Francisco': [
                'Alchemist', 'Bar Tartine', 'Lounge 3411', 'Tempest'
            ],
            'New York': [
                'Dead Rabbit', 'Death & Co.', 'Donna', 'Proletariat'
            ]
        }
        
        class City(self.env.Model):
            name = Text()
            bars = OneToMany("Bar", inverse="city", indexes=['name'])
            indexes = ['name']
        
        class Bar(self.env.Model):
            name = Text()
            city = ManyToOne(City, inverse="bars")
            indexes = ['name']
        
        with self.env.write():
            for city, bars in data.items():
                c = City(name=city)
                for bar in bars:
                    b = Bar(name=bar)
                    c.bars.add(b)
        
        with self.env.read():
            city = City.indexes['name'].get('San Francisco')
            self.assertEquals(city.bars.count(), 4)
            self.assertEquals([b.name for b in city.bars.indexes['name'].cursor()], data['San Francisco'])
            
            city = City.indexes['name'].get('New York')
            self.assertEquals(city.bars.count(), 4)
            self.assertEquals([b.name for b in city.bars.indexes['name'].cursor()], data['New York'])
            
            bar = Bar.indexes['name'].get('Donna')
            self.assertEquals(bar.city.name, 'New York')
            self.assertEquals(bar.city, city)
        
        with self.env.write():
            bar.remove()
            
        with self.env.read():
            self.assertEquals(city.bars.count(), 3)
            
            b = city.bars.first()
            name = b.name
            self.assertIn(name, data['New York'])
            
        with self.env.write():
            city.bars.remove(b)
            
        with self.env.read():
            self.assertEquals(city.bars.count(), 2)
            new_b = Bar.indexes['name'].get(name)
            self.assertEquals(b, new_b)
        

    def test_one_to_many_cascade(self):
        class City(self.env.Model):
            name = Text()
            bars = OneToMany("Bar", inverse="city", cascade=True)
        
        class Bar(self.env.Model):
            name = Text()
            indexes = ['name']
        
        with self.env.write():
            city_a = City(name="Foo City")
            city_b = City(name="Qux City")
            
            for i in range(0,10):
                a = Bar(name="Bar a#%s" % str(i+1))
                city_a.bars.add(a)
                
                b = Bar(name="Bar b#%s" % str(i+1))
                city_b.bars.add(b)
        
        with self.env.read():
            self.assertEquals(Bar.count(), 20)
            self.assertEquals(city_a.bars.count(), 10)
            self.assertEquals(city_b.bars.count(), 10)
            
        with self.env.write():
            city_a.remove()
            
        with self.env.read():
            self.assertEquals(Bar.count(), 10)
            b = Bar.indexes['name'].cursor().first()
            self.assertEquals(b.name, "Bar b#1")
        
        
    def test_many_to_many(self):
        tags_by_doc = {
            "foo": ["tofu", "seitan"],
            "bar": ["seitan"],
            "baz": ["tempeh", "tofu"],
            "qux": ["seitan", "tofu"],
            "goo": ["seitan"]
        }
        docs_by_tag = {}
        
        class Document(self.env.Model):
            content = Text()
            tags = ManyToMany("Tag", inverse="documents", indexes=['name'])
            indexes = ['content']
            
        class Tag(self.env.Model):
            name = Text()
            documents = ManyToMany(Document, inverse="tags", indexes=['content'])
            indexes = ['name']
        
        tags = {}
        docs = {}
        
        with self.env.write():
            for content, tag_names in tags_by_doc.items():
                doc = Document(content=content)
                docs[content] = doc
                
                for n in tag_names:
                    if n not in tags:
                        tag = Tag(name=n)
                        tags[n] = tag
                    doc.tags.add(tags[n])
                    if n not in docs_by_tag:
                        docs_by_tag[n] = []
                    docs_by_tag[n].append(content)
        
        for d in [tags_by_doc, docs_by_tag]:
            for k in d:
                d[k].sort()
        
        with self.env.read():
            for k,v in tags_by_doc.items():
                doc = Document.indexes['content'].get(k)
                self.assertEquals([tag.name for tag in doc.tags.indexes['name'].cursor()], v)
                
            for k,v in docs_by_tag.items():
                tag = Tag.indexes['name'].get(k)
                self.assertEquals([doc.content for doc in tag.documents.indexes['content'].cursor()], v)
        
            tag = Tag.indexes['name'].get('seitan')
            self.assertEquals(tag.documents.count(), 4)
            doc = tag.documents.first()
            self.assertIn('seitan', [tag.name for tag in doc.tags])
            num_doc_tags = doc.tags.count()
            
        with self.env.write():
            tag.documents.remove(doc)
            
        with self.env.read():
            self.assertEquals(tag.documents.count(), 3)
            self.assertEquals(doc.tags.count(), num_doc_tags-1)
            
            doc = Document.indexes['content'].get('baz')
            self.assertEquals(doc.tags.indexes['name'].cursor().count_key('tofu'), 1)
        

    def test_many_to_many_cascade(self):
        class Foo(self.env.Model):
            bars = ManyToMany("Bar", inverse="foos", cascade=True)
            
        class Bar(self.env.Model):
            pass
        
        foos = []
        
        with self.env.write():
            for i in range(0,3):
                f = Foo()
                for i in range(0,10):
                    b = Bar()
                    f.bars.add(b)
                foos.append(f)
        
        with self.env.read():
            self.assertEquals(Bar.count(), 30)
        
            for foo in foos:
                self.assertEquals(foo.bars.count(), 10)
        
        with self.env.write():
            foos[0].remove()
        
        with self.env.read():
            self.assertEquals(Bar.count(), 20)
            
            for foo in foos[1:]:
                self.assertEquals(foo.bars.count(), 10)

    def test_redefine_backreference_fails(self):
        class Foo(self.env.Model):
            pass
            
        class Bar(self.env.Model):
            foos = ManyToOne(Foo, inverse="bars")
        
        with self.assertRaises(AssertionError):
            class Baz(self.env.Model):
                foos = ManyToOne(Foo, inverse="bars")
        
        class Qux(self.env.Model):
            qugs = OneToMany("Qug", inverse="zzz")
        
        with self.assertRaises(AssertionError):    
            class Quv(self.env.Model):
                qugs = OneToMany("Qug", inverse="zzz")
                
                
    def test_invalid_inverse_fails(self):
        class Foo(self.env.Model):
            bar = OneToOne("Bar", inverse="foo")
        
        with self.assertRaises(TypeError):
            class Bar(self.env.Model):
                foo = OneToMany(Foo)
                
        
if __name__ == "__main__":
    unittest.main()



import itertools
import functools
import cursor
import dson


class FieldsGroup(object):
    
    def __init__(self, fields):
        field_names = []
        virtual_fields = []
        
        for f in fields:
            if isinstance(f, tuple):
                field_names.append(f[0])
                virtual_fields.append(f)
            else:
                field_names.append(f)
        
        self.names = tuple(field_names)
        self.virtual = dict(virtual_fields)
        
        
    def __getitem__(self, i):
        return self.names[i]
        
        
    def __iter__(self):
        return iter(self.names)
        
        
    def __str__(self):
        return str(self.names)
        
        
    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, str(self.names))
        
        
    def __eq__(self, other):
        if isinstance(other, FieldsGroup):
            return self.names == other.names
        elif isinstance(other, tuple):
            return other == self.names
        return False
        
        
    def __hash__(self):
        return hash(self.names)


class IndexCollection(object):
    
    def __init__(self, dbm, name):
        self.dbm = dbm
        self.name = name
        self.indexes = {}
        self.dbm.add_namespace('_indexes')
        
        
    def add(self, *fields, **kwargs):
        fields = FieldsGroup(fields)
        assert fields not in self.indexes, "Attempting to redefine index %s" % str(fields)
        self.indexes[fields] = Index(self.dbm, self.name, fields, 
            unique=kwargs.get('unique', False), filter=kwargs.get('filter', None))
        
        
    def __contains__(self, fields):
        if not isinstance(fields, tuple):
            fields = (fields,)
        return fields in self.indexes
        
        
    def get(self, fields):
        if not isinstance(fields, tuple):
            fields = (fields,)
        index = self.indexes.get(fields)
        if not index:
            raise KeyError, "No index for %s" % str(fields)
        return index
        
        
    def __getitem__(self, fields):
        return self.get(fields)
        
        
    def __iter__(self):
        for k in self.indexes:
            yield k
        
        
    def update(self, old_doc, new_doc):
        for index in self.indexes.values():
            index.update(old_doc, new_doc)
            
            
    def remove(self, doc):
        for index in self.indexes.values():
            index.remove(doc)
            
            
    def sync(self):
        self.dbm.transaction_start(writable=False)
        try:
            if self.dbm.count(self.name) == 0:
                self.dbm.transaction_commit()
                return
            
            data = self.dbm.get('_indexes', self.name)
        except:
            self.dbm.transaction_abort()
            raise
        else:
            self.dbm.transaction_commit()
            
            if data:
                field_groups = dson.loads(data)
            else:
                field_groups = []
            
            self.dbm.transaction_start(writable=True)
            
            try:
                indexes_to_sync = []
                new_field_groups = []
                for fields, index in self.indexes.items():
                    new_field_groups.append(fields.names)
                    if fields not in field_groups:
                        index.remove_all()
                        indexes_to_sync.append(index)
                
                self.dbm.put('_indexes', self.name, dson.dumps(new_field_groups))
                if len(indexes_to_sync) > 0:
                    c = cursor.Cursor(self.dbm, self.name)
                    for doc in c:
                        for index in indexes_to_sync:
                            index.update(None, doc)
            except:
                self.dbm.transaction_abort()
                raise
            else:
                self.dbm.transaction_commit()
        
        
    
class Index(object):
    
    def __init__(self, dbm, table_name, fields, unique=False, filter=None):
        self.dbm = dbm
        self.table_name = table_name
        self.fields = fields
        self.name = '%s.%s' % (self.table_name, ','.join(self.fields.names))
        self.dbm.add_namespace(self.name, duplicate_keys=(not unique))
        self.filter = filter
        
        
    def get_fields(self):
        return self.fields
        
        
    def update(self, old_doc, new_doc):
        if self.filter and not self.filter(new_doc):
            if old_doc and self.filter(old_doc):
                self.remove(old_doc)
            return
            
        value = dson.dumpone(new_doc['id'])
        new_keys = self.make_keys(new_doc)
        
        if old_doc:
            old_keys = self.make_keys(old_doc)
            if old_keys == new_keys:
                return
            for k in old_keys:
                self.dbm.delete(self.name, k, value)
        for k in new_keys:
            self.dbm.put(self.name, k, value)
        
        
    def get(self, key):
        string_key = self.get_key(key)
        value = self.dbm.get(self.name, string_key)
        if value:
            doc_value = self.dbm.get(self.table_name, value)
            if doc_value:
                return dson.loads(doc_value)
    
    def all(self, key):
        string_key = self.get_key(key)
        cur = self.dbm.cursor(self.name)
        cur.jump(string_key)
        while cur.key() == string_key:
            doc_key = cur.value()
            doc_value = self.dbm.get(self.table_name, doc_key)
            if doc_value:
                yield dson.loads(doc_value)
            cur.next()
        
        
    def remove(self, doc):
        key = self.get_key(doc)
        value = dson.dumpone(doc['id'])
        self.dbm.delete(self.name, key, value=value)
        
        
    def remove_all(self):
        self.dbm.delete_all(self.name)
        
        
    def cursor(self, reverse=False):
        return IndexCursor(self, reverse)
        
        
    def count(self):
        return self.dbm.count(self.name)
        
        
    def make_keys(self, doc):
        rows = []
        for f in self.fields.names:
            if f in self.fields.virtual:
                values = self.fields.virtual[f](doc)
                if len(values) > 0:
                    rows.append(values)
            else:
                try:
                    value = self.get_value(doc, f)
                except KeyError:
                    continue
                else:
                    if isinstance(value, list):
                        rows.append([value] + value)
                    else:
                        rows.append([value])
        keys = []
        for row in itertools.product(*rows):
            keys.append(self.dump_key(row))
        return keys
        
        
    def get_value(self, doc, field):
        if '.' in field:
            parts = field.split('.')
            child_doc = doc.get(parts[0])
            if not child_doc:
                raise KeyError, field
            return self.get_value(child_doc, '.'.join(parts[1:]))
        else:
            return doc.get(field)
        
        
    def get_key(self, doc):
        if isinstance(doc, dict):
            parts = []
            for f in self.fields:
                assert f in doc, "Missing value for field '%s' in index %s" % (f, str(self.fields))
                parts.append(doc[f])
        else:
            parts = (doc,)
            
        return self.dump_key(parts)
        
        
    def dump_key(self, parts):
        return ''.join(map(dson.dumpone, parts))
        
        
        
class IndexCursor(cursor.Cursor):
    
    def __init__(self, index, reverse=False):
        super(IndexCursor, self).__init__(index.dbm, index.name, reverse)
        self.index = index
        
        
    def dump_key(self, key):
        return self.index.get_key(key)
        
        
    def dump_prefix(self, prefix):
        parts = []
        for f in self.index.fields:
            if f not in prefix:
                break
            parts.append(prefix[f])
            
        assert len(parts) > 0, "Prefix is missing indexed fields or has out-of-order fields"
        return self.index.dump_key(parts)
        
        
    def load(self, data):
        doc_value = self.index.dbm.get(self.index.table_name, data)
        if doc_value:
            return dson.loads(doc_value)


def make_key(base_key, extension_fields, key=None):
    if key is None:
        key = {}
    if isinstance(key, dict):
        key.update(dict(base_key))
    else:
        if len(extension_fields) > 0:
            key = {
                extension_fields[0]: key
            }
            key.update(base_key)
        else:
            key = dict(base_key)
    return key


class BaseKeyIndexCollectionProxy(object):
    
    def __init__(self, index_collection, base_key):
        self._index_collection = index_collection
        self._base_key = base_key
        
        
    def add(self, *fields):
        complete_fields = self.get_complete_fields(fields)
        return self._index_collection.add(*complete_fields)
        
        
    def get(self, fields=tuple()):
        if not isinstance(fields,tuple):
            fields = (fields,)
        index = self._index_collection[self.get_complete_fields(fields)]
        return BaseKeyIndexProxy(index, self._base_key, fields)
        
    def __getitem__(self, fields):
        return self.get(fields)
        
        
    def __contains__(self, fields):
        return self.get_complete_fields(fields) in self._index_collection
        
        
    def __iter__(self):
        for fields in self._index_collection:
            yield fields[len(self._base_key):]
        
        
    def get_complete_fields(self, fields):
        base_fields = tuple([x[0] for x in self._base_key])
        if not isinstance(fields, tuple):
            fields = (fields,)
        return base_fields + fields
        
        
class BaseKeyIndexProxy(object):
    
    def __init__(self, index, base_key, extension_fields):
        self.index = index
        self.base_key = base_key
        self.extension_fields = extension_fields
        self.make_key = functools.partial(make_key, base_key, extension_fields)
        
        
    def get(self, key):
        return self.index.get(
            self.make_key(key)
        )
        
        
    def all(self, key):
        return self.index.all(
            self.make_key(key)
        )
        
        
    def count(self):
        return self.index.cursor().count_prefix(self.make_key())
        
        
    def cursor(self, reverse=False):
        return BaseKeyIndexCursorProxy(self.index.cursor(reverse=reverse), self.base_key, self.extension_fields)
            
            
class BaseKeyIndexCursorProxy(object):
    
    def __init__(self, cursor, base_key, extension_fields):
        self.cursor = cursor
        self.make_key = functools.partial(make_key, base_key, extension_fields)
        
        
    def first(self):
        try:
            self.cursor.prefix(self.make_key()).next()
        except StopIteration:
            pass
            
            
    def last(self):
        old_reverse = self.cursor.reverse
        self.cursor.reverse = True
        result = self.first()
        self.cursor.reverse = old_reverse
        return result
        
        
    def __iter__(self):
        return self.cursor.prefix(self.make_key())
        
        
    def range(self, start=None, end=None):
        start, end = [self.make_key(k) if k is not None else None for k in (start, end)]
        return self.cursor(start, end)
        
        
    def prefix(self, key):
        return self.cursor.prefix(self.make_key(key))
        
        
    def key(self, key):
        return self.cursor.key(self.make_key(key))
        
        
    def count_prefix(self, key):
        return self.cursor.count_prefix(self.make_key(key))
        
        
    def count_key(self, key):
        return self.cursor.count_key(self.make_key(key))
        